@@ -1,9 +1,11 @@
 #ifndef AUTH_CLIENT_CONNECTION_H
 #define AUTH_CLIENT_CONNECTION_H
 
+#include "master-auth.h"
+
 struct auth_client_connection {
+	struct auth_client_connection *prev, *next;
 	struct auth *auth;
-	struct auth_master_listener *listener;
 	int refcount;
 
 	int fd;
@@ -13,20 +15,20 @@ struct auth_client_connection {
 
 	unsigned int pid;
 	unsigned int connect_uid;
+	uint8_t cookie[MASTER_AUTH_COOKIE_SIZE];
 	struct auth_request_handler *request_handler;
 
+	unsigned int login_requests:1;
 	unsigned int version_received:1;
 };
 
 struct auth_client_connection *
-auth_client_connection_create(struct auth_master_listener *listener, int fd);
+auth_client_connection_create(struct auth *auth, int fd, bool login_requests);
 void auth_client_connection_destroy(struct auth_client_connection **conn);
 
 struct auth_client_connection *
-auth_client_connection_lookup(struct auth_master_listener *listener,
-			      unsigned int pid);
+auth_client_connection_lookup(unsigned int pid);
 
-void auth_client_connections_init(struct auth_master_listener *listener);
-void auth_client_connections_deinit(struct auth_master_listener *listener);
+void auth_client_connections_destroy_all(void);
 
 #endif
