@@ -100,7 +100,7 @@ master_input_request(struct auth_master_connection *conn, const char *args)
 	buffer_t buf;
 
 	/* <id> <client-pid> <client-id> <cookie> */
-	list = t_strsplit(args, "\t");
+	list = t_strsplit_tab(args);
 	if (str_array_length(list) < 4 ||
 	    str_to_uint(list[0], &id) < 0 ||
 	    str_to_uint(list[1], &client_pid) < 0 ||
@@ -146,7 +146,7 @@ master_input_auth_request(struct auth_master_connection *conn, const char *args,
 	unsigned int id;
 
 	/* <id> <userid> [<parameters>] */
-	list = t_strsplit(args, "\t");
+	list = t_strsplit_tab(args);
 	if (list[0] == NULL || list[1] == NULL ||
 	    str_to_uint(list[0], &id) < 0) {
 		i_error("BUG: Master sent broken %s", cmd);
@@ -485,7 +485,7 @@ master_input_list(struct auth_master_connection *conn, const char *args)
 	unsigned int id;
 
 	/* <id> [<parameters>] */
-	list = t_strsplit(args, "\t");
+	list = t_strsplit_tab(args);
 	if (list[0] == NULL || str_to_uint(list[0], &id) < 0) {
 		i_error("BUG: Master sent broken LIST");
 		return -1;
