@@ -237,6 +237,8 @@ bool auth_request_import_info(struct auth_request *request,
 		request->local_port = atoi(value);
 	else if (strcmp(key, "rport") == 0)
 		request->remote_port = atoi(value);
+	else if (strcmp(key, "session") == 0)
+		request->session_id = p_strdup(request->pool, value);
 	else
 		return FALSE;
 	return TRUE;
@@ -517,6 +519,10 @@ auth_request_handle_passdb_callback(enum passdb_result *result,
                 request->passdb = request->passdb->next;
 		request->passdb_password = NULL;
 
+		request->proxy = FALSE;
+		request->proxy_maybe = FALSE;
+		request->proxy_always = FALSE;
+
 		if (*result == PASSDB_RESULT_USER_UNKNOWN) {
 			/* remember that we did at least one successful
 			   passdb lookup */
@@ -551,6 +557,7 @@ auth_request_verify_plain_callback_finish(enum passdb_result result,
 			request->private_callback.verify_plain);
 	} else {
 		auth_request_ref(request);
+		request->passdb_result = result;
 		request->private_callback.verify_plain(result, request);
 		auth_request_unref(&request);
 	}
@@ -602,6 +609,20 @@ static bool password_has_illegal_chars(const char *password)
 	return FALSE;
 }
 
+static bool auth_request_is_disabled_master_user(struct auth_request *request)
+{
+	if (request->passdb != NULL)
+		return FALSE;
+
+	/* no masterdbs, master logins not supported */
+	i_assert(request->requested_login_user != NULL);
+	auth_request_log_info(request, "passdb",
+			      "Attempted master login with no master passdbs "
+			      "(trying to log in as user: %s)",
+			      request->requested_login_user);
+	return TRUE;
+}
+
 void auth_request_verify_plain(struct auth_request *request,
 			       const char *password,
 			       verify_plain_callback_t *callback)
@@ -612,13 +633,7 @@ void auth_request_verify_plain(struct auth_request *request,
 
 	i_assert(request->state == AUTH_REQUEST_STATE_MECH_CONTINUE);
 
-        if (request->passdb == NULL) {
-                /* no masterdbs, master logins not supported */
-                i_assert(request->requested_login_user != NULL);
-		auth_request_log_info(request, "passdb",
-			"Attempted master login with no master passdbs "
-			"(trying to log in as user: %s)",
-			request->requested_login_user);
+	if (auth_request_is_disabled_master_user(request)) {
 		callback(PASSDB_RESULT_USER_UNKNOWN, request);
 		return;
 	}
@@ -685,6 +700,7 @@ auth_request_lookup_credentials_finish(enum passdb_result result,
 			   but the user was unknown there */
 			result = PASSDB_RESULT_USER_UNKNOWN;
 		}
+		request->passdb_result = result;
 		request->private_callback.
 			lookup_credentials(result, credentials, size, request);
 	}
@@ -738,6 +754,11 @@ void auth_request_lookup_credentials(struct auth_request *request,
 
 	i_assert(request->state == AUTH_REQUEST_STATE_MECH_CONTINUE);
 
+	if (auth_request_is_disabled_master_user(request)) {
+		callback(PASSDB_RESULT_USER_UNKNOWN, NULL, 0, request);
+		return;
+	}
+
 	request->credentials_scheme = p_strdup(request->pool, scheme);
 	request->private_callback.lookup_credentials = callback;
 
@@ -1477,11 +1498,14 @@ static void auth_request_proxy_finish_ip(struct auth_request *request)
 	} else {
 		/* proxying to ourself - log in without proxying by dropping
 		   all the proxying fields. */
+		bool proxy_always = request->proxy_always;
+
 		auth_request_proxy_finish_failure(request);
-		if (request->proxy_always) {
+		if (proxy_always) {
 			/* director adds the host */
 			auth_stream_reply_add(request->extra_fields,
 					      "proxy", NULL);
+			request->proxy = TRUE;
 		}
 	}
 }
@@ -1514,8 +1538,8 @@ auth_request_proxy_dns_callback(const struct dns_lookup_result *result,
 				"DNS lookup for %s took %u.%03u s",
 				host, result->msecs/1000, result->msecs % 1000);
 		}
-		auth_stream_reply_remove(request->extra_fields, "host");
-		auth_stream_reply_add(request->extra_fields, "host",
+		auth_stream_reply_remove(request->extra_fields, "hostip");
+		auth_stream_reply_add(request->extra_fields, "hostip",
 				      net_ip2addr(&result->ips[0]));
 		for (i = 0; i < result->ips_count; i++) {
 			if (auth_request_proxy_ip_is_self(request,
@@ -1602,6 +1626,10 @@ void auth_request_proxy_finish_failure(struct auth_request *request)
 	auth_stream_reply_remove(request->extra_fields, "host");
 	auth_stream_reply_remove(request->extra_fields, "port");
 	auth_stream_reply_remove(request->extra_fields, "destuser");
+
+	request->proxy = FALSE;
+	request->proxy_maybe = FALSE;
+	request->proxy_always = FALSE;
 }
 
 static void log_password_failure(struct auth_request *request,
@@ -1760,6 +1788,7 @@ const struct var_expand_table auth_request_var_expand_static_tab[] = {
 	{ '\0', NULL, "login_user" },
 	{ '\0', NULL, "login_username" },
 	{ '\0', NULL, "login_domain" },
+	{ '\0', NULL, "session" },
 	{ '\0', NULL, NULL }
 };
 
@@ -1819,6 +1848,8 @@ auth_request_get_var_expand_table(const struct auth_request *auth_request,
 						    auth_request);
 		}
 	}
+	tab[18].value = auth_request->session_id == NULL ? NULL :
+		escape_func(auth_request->session_id, auth_request);
 	return tab;
 }
 
@@ -1845,6 +1876,8 @@ static void get_log_prefix(string_t *str, struct auth_request *auth_request,
 	}
 	if (auth_request->requested_login_user != NULL)
 		str_append(str, ",master");
+	if (auth_request->session_id != NULL)
+		str_printfa(str, ",<%s>", auth_request->session_id);
 	str_append(str, "): ");
 }
 
