@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
 
 #include "auth-common.h"
 #include "array.h"
@@ -9,6 +9,7 @@
 #include "child-wait.h"
 #include "sql-api.h"
 #include "module-dir.h"
+#include "hostpid.h"
 #include "randgen.h"
 #include "process-title.h"
 #include "settings-parser.h"
@@ -25,8 +26,10 @@
 #include "auth-worker-client.h"
 #include "auth-master-connection.h"
 #include "auth-client-connection.h"
+#include "auth-postfix-connection.h"
 
 #include <unistd.h>
+#include <sys/stat.h>
 
 #define AUTH_PENALTY_ANVIL_PATH "anvil-auth-penalty"
 
@@ -35,30 +38,32 @@ enum auth_socket_type {
 	AUTH_SOCKET_CLIENT,
 	AUTH_SOCKET_LOGIN_CLIENT,
 	AUTH_SOCKET_MASTER,
-	AUTH_SOCKET_USERDB
+	AUTH_SOCKET_USERDB,
+	AUTH_SOCKET_POSTFIX
 };
 
-bool worker = FALSE, shutdown_request = FALSE;
+struct auth_socket_listener {
+	enum auth_socket_type type;
+	struct stat st;
+	char *path;
+};
+
+bool worker = FALSE, worker_restart_request = FALSE;
 time_t process_start_time;
 struct auth_penalty *auth_penalty;
 
 static pool_t auth_set_pool;
 static struct module *modules = NULL;
 static struct mechanisms_register *mech_reg;
-static ARRAY_DEFINE(listen_fd_types, enum auth_socket_type);
+static ARRAY_DEFINE(listeners, struct auth_socket_listener);
 
 void auth_refresh_proctitle(void)
 {
-	const char *auth_worker_prefix;
-
-	if (!global_auth_settings->verbose_proctitle)
+	if (!global_auth_settings->verbose_proctitle || worker)
 		return;
 
-	auth_worker_prefix = worker ? "worker " : "";
-
 	process_title_set(t_strdup_printf(
-		"%s[%u wait, %u passdb, %u userdb]",
-		auth_worker_prefix,
+		"[%u wait, %u passdb, %u userdb]",
 		auth_request_state_count[AUTH_REQUEST_STATE_NEW] +
 		auth_request_state_count[AUTH_REQUEST_STATE_MECH_CONTINUE] +
 		auth_request_state_count[AUTH_REQUEST_STATE_FINISHED],
@@ -87,6 +92,72 @@ static const char *const *read_global_settings(void)
 	return services;
 }
 
+static enum auth_socket_type
+auth_socket_type_get(const char *path)
+{
+	const char *name, *suffix;
+
+	name = strrchr(path, '/');
+	if (name == NULL)
+		name = path;
+	else
+		name++;
+
+	suffix = strrchr(name, '-');
+	if (suffix == NULL)
+		suffix = name;
+	else
+		suffix++;
+
+	if (strcmp(suffix, "login") == 0)
+		return AUTH_SOCKET_LOGIN_CLIENT;
+	else if (strcmp(suffix, "master") == 0)
+		return AUTH_SOCKET_MASTER;
+	else if (strcmp(suffix, "userdb") == 0)
+		return AUTH_SOCKET_USERDB;
+	else if (strcmp(suffix, "postmap") == 0)
+		return AUTH_SOCKET_POSTFIX;
+	else
+		return AUTH_SOCKET_CLIENT;
+}
+
+static void listeners_init(void)
+{
+	unsigned int i, n;
+	const char *path;
+
+	i_array_init(&listeners, 8);
+	n = master_service_get_socket_count(master_service);
+	for (i = 0; i < n; i++) {
+		int fd = MASTER_LISTEN_FD_FIRST + i;
+		struct auth_socket_listener *l;
+
+		l = array_idx_modifiable(&listeners, fd);
+		if (net_getunixname(fd, &path) < 0) {
+			if (errno != ENOTSOCK)
+				i_fatal("getunixname(%d) failed: %m", fd);
+			/* not a unix socket, set its name and type lazily */
+		} else {
+			l->type = auth_socket_type_get(path);
+			l->path = i_strdup(path);
+			if (l->type == AUTH_SOCKET_USERDB) {
+				if (stat(path, &l->st) < 0)
+					i_error("stat(%s) failed: %m", path);
+			}
+		}
+	}
+}
+
+static bool auth_module_filter(const char *name, void *context ATTR_UNUSED)
+{
+	if (strncmp(name, "authdb_", 7) == 0 ||
+	    strncmp(name, "mech_", 5) == 0) {
+		/* this is lazily loaded */
+		return FALSE;
+	}
+	return TRUE;
+}
+
 static void main_preinit(void)
 {
 	struct module_dir_load_settings mod_set;
@@ -112,6 +183,7 @@ static void main_preinit(void)
 	mod_set.version = master_service_get_version_string(master_service);
 	mod_set.require_init_funcs = TRUE;
 	mod_set.debug = global_auth_settings->debug;
+	mod_set.filter_callback = auth_module_filter;
 
 	modules = module_dir_load(AUTH_MODULE_DIR, NULL, &mod_set);
 	module_dir_init(modules);
@@ -123,27 +195,44 @@ static void main_preinit(void)
 	auths_preinit(global_auth_settings, auth_set_pool,
 		      mech_reg, services);
 
+	listeners_init();
+
 	/* Password lookups etc. may require roots, allow it. */
 	restrict_access_by_env(NULL, FALSE);
 	restrict_access_allow_coredumps(TRUE);
 }
 
+void auth_module_load(const char *names)
+{
+	struct module_dir_load_settings mod_set;
+
+	memset(&mod_set, 0, sizeof(mod_set));
+	mod_set.version = master_service_get_version_string(master_service);
+	mod_set.require_init_funcs = TRUE;
+	mod_set.debug = global_auth_settings->debug;
+	mod_set.ignore_missing = TRUE;
+
+	modules = module_dir_load_missing(modules, AUTH_MODULE_DIR, names,
+					  &mod_set);
+	module_dir_init(modules);
+}
+
 static void main_init(void)
 {
-	i_array_init(&listen_fd_types, 8);
-
         process_start_time = ioloop_time;
 
 	/* If auth caches aren't used, just ignore these signals */
 	lib_signals_ignore(SIGHUP, TRUE);
 	lib_signals_ignore(SIGUSR2, TRUE);
 
+	/* set proctitles before init()s, since they may set them to error */
+	auth_refresh_proctitle();
+	auth_worker_refresh_proctitle(NULL);
+
 	child_wait_init();
 	auth_worker_server_init();
 	auths_init();
 	auth_request_handler_init();
-	auth_master_connections_init();
-	auth_client_connections_init();
 
 	if (worker) {
 		/* workers have only a single connection from the master
@@ -153,11 +242,12 @@ static void main_init(void)
 		/* caching is handled only by the main auth process */
 		passdb_cache_init(global_auth_settings);
 	}
-	auth_refresh_proctitle();
 }
 
 static void main_deinit(void)
 {
+	struct auth_socket_listener *l;
+
 	if (auth_penalty != NULL) {
 		/* cancel all pending anvil penalty lookups */
 		auth_penalty_deinit(&auth_penalty);
@@ -171,8 +261,9 @@ static void main_deinit(void)
 	/* there are no more auth requests */
 	auths_free();
 
-	auth_client_connections_deinit();
-	auth_master_connections_deinit();
+	auth_client_connections_destroy_all();
+	auth_master_connections_destroy_all();
+	auth_postfix_connections_destroy_all();
 
 	if (auth_worker_client != NULL)
 		auth_worker_client_destroy(&auth_worker_client);
@@ -190,8 +281,11 @@ static void main_deinit(void)
         password_schemes_deinit();
 	sql_drivers_deinit();
 	random_deinit();
+	child_wait_deinit();
 
-	array_free(&listen_fd_types);
+	array_foreach_modifiable(&listeners, l)
+		i_free(l->path);
+	array_free(&listeners);
 	pool_unref(&auth_set_pool);
 }
 
@@ -206,59 +300,30 @@ static void worker_connected(struct master_service_connection *conn)
 	(void)auth_worker_client_create(auth_find_service(NULL), conn->fd);
 }
 
-static enum auth_socket_type
-auth_socket_type_get(int listen_fd)
-{
-	const char *path, *name, *suffix;
-
-	/* figure out if this is a server or network socket by
-	   checking the socket path name. */
-	if (net_getunixname(listen_fd, &path) < 0) {
-		if (errno != ENOTSOCK)
-			i_fatal("getunixname(%d) failed: %m", listen_fd);
-		/* not UNIX socket. let's just assume it's an
-		   auth client. */
-		return AUTH_SOCKET_CLIENT;
-	}
-
-	name = strrchr(path, '/');
-	if (name == NULL)
-		name = path;
-	else
-		name++;
-
-	suffix = strrchr(name, '-');
-	if (suffix == NULL)
-		suffix = name;
-	else
-		suffix++;
-
-	if (strcmp(suffix, "login") == 0)
-		return AUTH_SOCKET_LOGIN_CLIENT;
-	else if (strcmp(suffix, "master") == 0)
-		return AUTH_SOCKET_MASTER;
-	else if (strcmp(suffix, "userdb") == 0)
-		return AUTH_SOCKET_USERDB;
-	else
-		return AUTH_SOCKET_CLIENT;
-}
-
 static void client_connected(struct master_service_connection *conn)
 {
-	enum auth_socket_type *type;
+	struct auth_socket_listener *l;
 	struct auth *auth;
 
-	type = array_idx_modifiable(&listen_fd_types, conn->listen_fd);
-	if (*type == AUTH_SOCKET_UNKNOWN)
-		*type = auth_socket_type_get(conn->listen_fd);
-
+	l = array_idx_modifiable(&listeners, conn->listen_fd);
+	if (l->type == AUTH_SOCKET_UNKNOWN) {
+		/* first connection from inet socket, figure out its type
+		   from the listener name */
+		l->type = auth_socket_type_get(conn->name);
+		l->path = i_strdup(conn->name);
+	}
 	auth = auth_find_service(NULL);
-	switch (*type) {
+	switch (l->type) {
 	case AUTH_SOCKET_MASTER:
-		(void)auth_master_connection_create(auth, conn->fd, FALSE);
+		(void)auth_master_connection_create(auth, conn->fd,
+						    l->path, NULL, FALSE);
 		break;
 	case AUTH_SOCKET_USERDB:
-		(void)auth_master_connection_create(auth, conn->fd, TRUE);
+		(void)auth_master_connection_create(auth, conn->fd,
+						    l->path, &l->st, TRUE);
+		break;
+	case AUTH_SOCKET_POSTFIX:
+		(void)auth_postfix_connection_create(auth, conn->fd);
 		break;
 	case AUTH_SOCKET_LOGIN_CLIENT:
 		(void)auth_client_connection_create(auth, conn->fd, TRUE);
@@ -274,7 +339,12 @@ static void client_connected(struct master_service_connection *conn)
 
 static void auth_die(void)
 {
-	/* do nothing. auth clients should disconnect soon. */
+	if (!worker) {
+		/* do nothing. auth clients should disconnect soon. */
+	} else {
+		/* ask auth master to disconnect us */
+		auth_worker_client_send_shutdown();
+	}
 }
 
 int main(int argc, char *argv[])
@@ -288,7 +358,7 @@ int main(int argc, char *argv[])
 		switch (c) {
 		case 'w':
 			master_service_init_log(master_service,
-						"auth-worker: ");
+				t_strdup_printf("auth-worker(%s): ", my_pid));
 			worker = TRUE;
 			break;
 		default:
