@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
 
 #include "auth-common.h"
 #include "passdb.h"
@@ -57,16 +57,21 @@ shadow_verify_plain(struct auth_request *request, const char *password,
 	callback(PASSDB_RESULT_OK, request);
 }
 
-static void shadow_init(struct passdb_module *module)
+static struct passdb_module *
+shadow_preinit(pool_t pool, const char *args)
 {
+	struct passdb_module *module;
+
+	module = p_new(pool, struct passdb_module, 1);
 	module->blocking = TRUE;
-	if (strcmp(module->args, "blocking=no") == 0)
+	if (strcmp(args, "blocking=no") == 0)
 		module->blocking = FALSE;
-	else if (*module->args != '\0')
-		i_fatal("passdb shadow: Unknown setting: %s", module->args);
+	else if (*args != '\0')
+		i_fatal("passdb shadow: Unknown setting: %s", args);
 
 	module->cache_key = SHADOW_CACHE_KEY;
 	module->default_pass_scheme = SHADOW_PASS_SCHEME;
+	return module;
 }
 
 static void shadow_deinit(struct passdb_module *module ATTR_UNUSED)
@@ -77,8 +82,8 @@ static void shadow_deinit(struct passdb_module *module ATTR_UNUSED)
 struct passdb_module_interface passdb_shadow = {
 	"shadow",
 
+	shadow_preinit,
 	NULL,
-	shadow_init,
 	shadow_deinit,
 
 	shadow_verify_plain,
