@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2005-2012 Dovecot authors, see the included COPYING file */
 
 #include "auth-common.h"
 #include "str.h"
@@ -10,7 +10,6 @@
 
 struct blocking_userdb_iterate_context {
 	struct userdb_iterate_context ctx;
-	pool_t pool;
 	struct auth_worker_connection *conn;
 	bool next;
 	bool destroyed;
@@ -40,8 +39,7 @@ static bool user_callback(const char *reply, void *context)
 	if (*args != '\0') {
 		request->userdb_reply = auth_stream_reply_init(request->pool);
 		auth_stream_reply_import(request->userdb_reply, args);
-		if (auth_stream_reply_find(request->userdb_reply,
-					   "tempfail") != NULL)
+		if (auth_stream_reply_exists(request->userdb_reply, "tempfail"))
 			request->userdb_lookup_failed = TRUE;
 	}
 
@@ -66,7 +64,6 @@ void userdb_blocking_lookup(struct auth_request *request)
 static bool iter_callback(const char *reply, void *context)
 {
 	struct blocking_userdb_iterate_context *ctx = context;
-	pool_t pool = ctx->pool;
 
 	if (strncmp(reply, "*\t", 2) == 0) {
 		ctx->next = FALSE;
@@ -78,31 +75,30 @@ static bool iter_callback(const char *reply, void *context)
 		ctx->ctx.failed = TRUE;
 	if (!ctx->destroyed)
 		ctx->ctx.callback(NULL, ctx->ctx.context);
-	pool_unref(&pool);
+	auth_request_unref(&ctx->ctx.auth_request);
 	return TRUE;
 }
 
 struct userdb_iterate_context *
-userdb_blocking_iter_init(struct userdb_module *userdb,
+userdb_blocking_iter_init(struct auth_request *request,
 			  userdb_iter_callback_t *callback, void *context)
 {
 	struct blocking_userdb_iterate_context *ctx;
 	struct auth_stream_reply *reply;
-	pool_t pool;
 
 	reply = auth_stream_reply_init(pool_datastack_create());
 	auth_stream_reply_add(reply, "LIST", NULL);
-	auth_stream_reply_add(reply, NULL, dec2str(userdb->id));
+	auth_stream_reply_add(reply, NULL,
+			      dec2str(request->userdb->userdb->id));
+	auth_request_export(request, reply);
 
-	pool = pool_alloconly_create("userdb iter", 512);
-	ctx = p_new(pool, struct blocking_userdb_iterate_context, 1);
-	ctx->ctx.userdb = userdb;
+	ctx = p_new(request->pool, struct blocking_userdb_iterate_context, 1);
+	ctx->ctx.auth_request = request;
 	ctx->ctx.callback = callback;
 	ctx->ctx.context = context;
-	ctx->pool = pool;
 
-	pool_ref(pool);
-	ctx->conn = auth_worker_call(pool, reply, iter_callback, ctx);
+	auth_request_ref(request);
+	ctx->conn = auth_worker_call(request->pool, reply, iter_callback, ctx);
 	return &ctx->ctx;
 }
 
@@ -123,7 +119,7 @@ int userdb_blocking_iter_deinit(struct userdb_iterate_context **_ctx)
 
 	*_ctx = NULL;
 
+	/* iter_callback() may still be called */
 	ctx->destroyed = TRUE;
-	pool_unref(&ctx->pool);
 	return ret;
 }
