@@ -1,4 +1,4 @@
-/* Copyright (c) 2003-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2003-2012 Dovecot authors, see the included COPYING file */
 
 #include "auth-common.h"
 #include "userdb.h"
@@ -6,7 +6,7 @@
 #if defined(USERDB_LDAP) && (defined(BUILTIN_LDAP) || defined(PLUGIN_BUILD))
 
 #include "ioloop.h"
-#include "hash.h"
+#include "array.h"
 #include "str.h"
 #include "var-expand.h"
 #include "auth-cache.h"
@@ -51,11 +51,12 @@ ldap_query_get_result(struct ldap_connection *conn, LDAPMessage *entry,
 	auth_request_init_userdb_reply(auth_request);
 
 	ldap_iter = db_ldap_result_iterate_init(conn, entry, auth_request,
-						conn->user_attr_map);
+						&conn->user_attr_map);
 	while (db_ldap_result_iterate_next(ldap_iter, &name, &values)) {
 		auth_request_set_userdb_field_values(auth_request,
 						     name, values);
 	}
+	db_ldap_result_iterate_deinit(&ldap_iter);
 }
 
 static void
@@ -167,7 +168,7 @@ static void userdb_ldap_iterate_callback(struct ldap_connection *conn,
 	ctx->in_callback = TRUE;
 	ldap_iter = db_ldap_result_iterate_init(conn, res,
 						request->auth_request,
-						conn->iterate_attr_map);
+						&conn->iterate_attr_map);
 	while (db_ldap_result_iterate_next(ldap_iter, &name, &values)) {
 		if (strcmp(name, "user") != 0) {
 			i_warning("ldap: iterate: "
@@ -179,48 +180,51 @@ static void userdb_ldap_iterate_callback(struct ldap_connection *conn,
 			ctx->ctx.callback(*values, ctx->ctx.context);
 		}
 	}
+	db_ldap_result_iterate_deinit(&ldap_iter);
 	if (!ctx->continued)
 		db_ldap_enable_input(conn, FALSE);
 	ctx->in_callback = FALSE;
 }
 
 static struct userdb_iterate_context *
-userdb_ldap_iterate_init(struct userdb_module *userdb,
+userdb_ldap_iterate_init(struct auth_request *auth_request,
 			 userdb_iter_callback_t *callback, void *context)
 {
-	static struct var_expand_table static_tab[] = {
-		/* nothing for now, but e.g. %{hostname} can be used */
-		{ '\0', NULL, NULL }
-	};
+	struct userdb_module *_module = auth_request->userdb->userdb;
 	struct ldap_userdb_module *module =
-		(struct ldap_userdb_module *)userdb;
+		(struct ldap_userdb_module *)_module;
 	struct ldap_connection *conn = module->conn;
 	struct ldap_userdb_iterate_context *ctx;
 	struct userdb_iter_ldap_request *request;
+        const struct var_expand_table *vars;
 	const char **attr_names = (const char **)conn->iterate_attr_names;
 	string_t *str;
 
 	ctx = i_new(struct ldap_userdb_iterate_context, 1);
-	ctx->ctx.userdb = userdb;
+	ctx->ctx.auth_request = auth_request;
 	ctx->ctx.callback = callback;
 	ctx->ctx.context = context;
 	ctx->conn = conn;
 	request = &ctx->request;
 	request->ctx = ctx;
 
-	request->request.request.auth_request = auth_request_new_dummy();
-	request->request.base = conn->set.base;
+	auth_request_ref(auth_request);
+	request->request.request.auth_request = auth_request;
+
+	vars = auth_request_get_var_expand_table(auth_request, ldap_escape);
 
 	str = t_str_new(512);
-	var_expand(str, conn->set.iterate_filter, static_tab);
-	request->request.filter =
-		p_strdup(request->request.request.auth_request->pool,
-			 str_c(str));
+	var_expand(str, conn->set.base, vars);
+	request->request.base = p_strdup(auth_request->pool, str_c(str));
+
+	str_truncate(str, 0);
+	var_expand(str, conn->set.iterate_filter, vars);
+	request->request.filter = p_strdup(auth_request->pool, str_c(str));
 	request->request.attributes = conn->iterate_attr_names;
 
 	if (global_auth_settings->debug) {
 		i_debug("ldap: iterate: base=%s scope=%s filter=%s fields=%s",
-			conn->set.base, conn->set.scope,
+			request->request.base, conn->set.scope,
 			request->request.filter, attr_names == NULL ? "(all)" :
 			t_strarray_join(attr_names, ","));
 	}
@@ -258,19 +262,15 @@ userdb_ldap_preinit(pool_t pool, const char *args)
 	struct ldap_connection *conn;
 
 	module = p_new(pool, struct ldap_userdb_module, 1);
-	module->conn = conn = db_ldap_init(args);
-	conn->user_attr_map =
-		hash_table_create(default_pool, conn->pool, 0, strcase_hash,
-				  (hash_cmp_callback_t *)strcasecmp);
-	conn->iterate_attr_map =
-		hash_table_create(default_pool, conn->pool, 0, strcase_hash,
-				  (hash_cmp_callback_t *)strcasecmp);
+	module->conn = conn = db_ldap_init(args, TRUE);
+	p_array_init(&conn->user_attr_map, pool, 16);
+	p_array_init(&conn->iterate_attr_map, pool, 16);
 
 	db_ldap_set_attrs(conn, conn->set.user_attrs, &conn->user_attr_names,
-			  conn->user_attr_map, NULL);
+			  &conn->user_attr_map, NULL);
 	db_ldap_set_attrs(conn, conn->set.iterate_attrs,
 			  &conn->iterate_attr_names,
-			  conn->iterate_attr_map, NULL);
+			  &conn->iterate_attr_map, NULL);
 	module->module.cache_key =
 		auth_cache_parse_key(pool,
 				     t_strconcat(conn->set.base,
