@@ -287,14 +287,17 @@ config_module_parser_apply_changes(struct config_module_parser *dest,
 				   const struct config_filter_parser *src,
 				   pool_t pool, const char **error_r)
 {
+	const char *conflict_key;
 	unsigned int i;
 
 	for (i = 0; dest[i].root != NULL; i++) {
 		if (settings_parser_apply_changes(dest[i].parser,
 						  src->parsers[i].parser, pool,
-						  error_r) < 0) {
+						  error_r == NULL ? NULL :
+						  &conflict_key) < 0) {
+			i_assert(error_r != NULL);
 			*error_r = t_strdup_printf("Conflict in setting %s "
-				"found from filter at %s", *error_r,
+				"found from filter at %s", conflict_key,
 				src->file_and_line);
 			return -1;
 		}
@@ -314,6 +317,11 @@ int config_filter_parsers_get(struct config_filter_context *ctx, pool_t pool,
 	const char *error = NULL, **error_p;
 	unsigned int i, count;
 
+	/* get the matching filters. the most specific ones are handled first,
+	   so that if more generic filters try to override settings we'll fail
+	   with an error. Merging SET_STRLIST types requires
+	   settings_parser_apply_changes() to work a bit unintuitively by
+	   letting the destination settings override the source settings. */
 	src = config_filter_find_all(ctx, module, filter, output_r);
 
 	/* all of them should have the same number of parsers.
