@@ -14,21 +14,19 @@ void director_connection_deinit(struct director_connection **conn);
 
 void director_connection_send(struct director_connection *conn,
 			      const char *data);
-void director_connection_wait_sync(struct director_connection *conn);
-void director_connection_send_except(struct director_connection *conn,
-				     struct director_host *skip_host,
-				     const char *data);
+void director_connection_set_synced(struct director_connection *conn,
+				    bool synced);
+void director_connection_ping(struct director_connection *conn);
 
 const char *director_connection_get_name(struct director_connection *conn);
 struct director_host *
 director_connection_get_host(struct director_connection *conn);
-struct director_connection *
-director_connection_find_outgoing(struct director *dir,
-				  struct director_host *host);
+bool director_connection_is_handshaked(struct director_connection *conn);
+bool director_connection_is_incoming(struct director_connection *conn);
+unsigned int
+director_connection_get_minor_version(struct director_connection *conn);
 
 void director_connection_cork(struct director_connection *conn);
 void director_connection_uncork(struct director_connection *conn);
 
-void director_connections_deinit(struct director *dir);
-
 #endif
