@@ -6,10 +6,16 @@
 
 #define DIRECTOR_VERSION_NAME "director"
 #define DIRECTOR_VERSION_MAJOR 1
-#define DIRECTOR_VERSION_MINOR 1
+#define DIRECTOR_VERSION_MINOR 2
 
 /* weak users supported in protocol v1.1+ */
 #define DIRECTOR_VERSION_WEAK_USERS 1
+/* director removes supported in v1.2+ */
+#define DIRECTOR_VERSION_RING_REMOVE 2
+
+/* Minimum time between even attempting to communicate with a director that
+   failed due to a protocol error. */
+#define DIRECTOR_PROTOCOL_FAILURE_RETRY_SECS 60
 
 struct director;
 struct mail_host;
@@ -27,9 +33,13 @@ struct director {
 	unsigned int test_port;
 
 	struct director_host *self_host;
+	/* left and right connections are set only after they have finished
+	   handshaking. until then they're in the connections list, although
+	   updates are still sent to them during handshaking if the USER list
+	   is long. */
 	struct director_connection *left, *right;
 	/* all director connections */
-	struct director_connection *connections;
+	ARRAY_DEFINE(connections, struct director_connection *);
 	struct timeout *to_reconnect;
 	struct timeout *to_sync;
 
@@ -50,6 +60,7 @@ struct director {
 
 	/* director hosts are sorted by IP (and port) */
 	ARRAY_DEFINE(dir_hosts, struct director_host *);
+	struct timeout *to_remove_dirs;
 
 	struct ipc_client *ipc_proxy;
 	unsigned int sync_seq;
@@ -57,6 +68,8 @@ struct director {
 	unsigned int ring_min_version;
 	time_t ring_last_sync_time;
 
+	time_t ring_first_alone;
+
 	/* director ring handshaking is complete.
 	   director can start serving clients. */
 	unsigned int ring_handshaked:1;
@@ -89,6 +102,11 @@ void director_sync_send(struct director *dir, struct director_host *host,
 			uint32_t seq, unsigned int minor_version);
 bool director_resend_sync(struct director *dir);
 
+void director_notify_ring_added(struct director_host *added_host,
+				struct director_host *src);
+void director_ring_remove(struct director_host *removed_host,
+			  struct director_host *src);
+
 void director_update_host(struct director *dir, struct director_host *src,
 			  struct director_host *orig_src,
 			  struct mail_host *host);
@@ -119,7 +137,10 @@ void director_sync_thaw(struct director *dir);
 /* Send data to all directors using both left and right connections
    (unless they're the same). */
 void director_update_send(struct director *dir, struct director_host *src,
-			  const char *data);
+			  const char *cmd);
+void director_update_send_version(struct director *dir,
+				  struct director_host *src,
+				  unsigned int min_version, const char *cmd);
 
 int director_connect_host(struct director *dir, struct director_host *host);
 
