@@ -121,7 +121,7 @@ static void auth_input_line(const char *line, void *context)
 	}
 
 	/* OK <id> [<parameters>] */
-	args = t_strsplit(line_params, "\t");
+	args = t_strsplit_tab(line_params);
 	if (*args != NULL) {
 		/* we should always get here, but in case we don't just
 		   forward as-is and let login process handle the error. */
