@@ -139,12 +139,18 @@ doveadm_mail_mailbox_search_args_build(const char *const args[])
 {
 	struct mail_search_args *search_args;
 	struct mail_search_arg *arg;
+	enum mail_search_arg_type type;
 	unsigned int i;
 
 	doveadm_mailbox_args_check(args);
 	search_args = mail_search_build_init();
 	for (i = 0; args[i] != NULL; i++) {
-		arg = mail_search_build_add(search_args, SEARCH_MAILBOX_GLOB);
+		if (strchr(args[i], '*') != NULL ||
+		    strchr(args[i], '%') != NULL)
+			type = SEARCH_MAILBOX_GLOB;
+		else
+			type = SEARCH_MAILBOX;
+		arg = mail_search_build_add(search_args, type);
 		arg->value.str = p_strdup(search_args->pool, args[i]);
 	}
 	if (i > 1) {
@@ -371,6 +377,9 @@ cmd_mailbox_delete_parse_arg(struct doveadm_mail_cmd_context *_ctx, int c)
 	case 'r':
 		ctx->recursive = TRUE;
 		break;
+	case 's':
+		ctx->ctx.subscriptions = TRUE;
+		break;
 	default:
 		return FALSE;
 	}
@@ -385,7 +394,7 @@ static struct doveadm_mail_cmd_context *cmd_mailbox_delete_alloc(void)
 	ctx->ctx.ctx.v.init = cmd_mailbox_delete_init;
 	ctx->ctx.ctx.v.run = cmd_mailbox_delete_run;
 	ctx->ctx.ctx.v.parse_arg = cmd_mailbox_delete_parse_arg;
-	ctx->ctx.ctx.getopt_args = "r";
+	ctx->ctx.ctx.getopt_args = "rs";
 	p_array_init(&ctx->mailboxes, ctx->ctx.ctx.pool, 16);
 	return &ctx->ctx.ctx;
 }
