@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -103,8 +103,15 @@ static void doveadm_calc_header_length(void)
 			break;
 		}
 	}
-	if (max_length < ctx->columns)
-		headers[0].length += (ctx->columns - max_length) / 2;
+	if (max_length < ctx->columns) {
+		for (i = 0; i < hdr_count; i++) {
+			if ((headers[i].flags & DOVEADM_PRINT_HEADER_FLAG_EXPAND) != 0) {
+				i++;
+				break;
+			}
+		}
+		headers[i-1].length += (ctx->columns - max_length) / 2;
+	}
 }
 
 static void doveadm_print_next(const char *value)
@@ -189,7 +196,7 @@ static void doveadm_print_table_init(void)
 	pool_t pool;
 	struct winsize ws;
 
-	pool = pool_alloconly_create("doveadm print table", 1024);
+	pool = pool_alloconly_create("doveadm print table", 2048);
 	ctx = p_new(pool, struct doveadm_print_table_context, 1);
 	ctx->pool = pool;
 	p_array_init(&ctx->headers, pool, 16);
