@@ -21,16 +21,18 @@ static void cmd_pw(int argc, char *argv[])
 	const char *user = NULL;
 	const char *scheme = NULL;
 	const char *plaintext = NULL;
-	int ch, lflag = 0, Vflag = 0;
+	const char *test_hash = NULL;
+	bool list_schemes = FALSE, reverse_verify = FALSE;
 	unsigned int rounds = 0;
+	int c;
 
 	random_init();
 	password_schemes_init();
 	
-	while ((ch = getopt(argc, argv, "lp:r:s:u:V")) != -1) {
-		switch (ch) {
+	while ((c = getopt(argc, argv, "lp:r:s:t:u:V")) > 0) {
+		switch (c) {
 		case 'l':
-			lflag = 1;
+			list_schemes = 1;
 			break;
 		case 'p':
 			plaintext = optarg;
@@ -42,11 +44,15 @@ static void cmd_pw(int argc, char *argv[])
 		case 's':
 			scheme = optarg;
 			break;
+		case 't':
+			test_hash = optarg;
+			reverse_verify = TRUE;
+			break;
 		case 'u':
 			user = optarg;
 			break;
 		case 'V':
-			Vflag = 1;
+			reverse_verify = TRUE;
 			break;
 		case '?':
 		default:
@@ -54,7 +60,7 @@ static void cmd_pw(int argc, char *argv[])
 		}
 	}
 
-	if (lflag) {
+	if (list_schemes) {
 		const struct password_scheme *const *schemes;
 		unsigned int i, count;
 
@@ -72,6 +78,8 @@ static void cmd_pw(int argc, char *argv[])
 	if (rounds > 0)
 		password_set_encryption_rounds(rounds);
 
+	if (test_hash != NULL)
+		plaintext = t_askpass("Enter password to verify: ");
 	while (plaintext == NULL) {
 		const char *check;
 		static int lives = 3;
@@ -79,38 +87,41 @@ static void cmd_pw(int argc, char *argv[])
 		plaintext = t_askpass("Enter new password: ");
 		check = t_askpass("Retype new password: ");
 		if (strcmp(plaintext, check) != 0) {
-			fprintf(stderr, "Passwords don't match!\n");
+			i_error("Passwords don't match!");
 			if (--lives == 0)
 				exit(1);
 			plaintext = NULL;
 		}
 	}
 
-	if (!password_generate_encoded(plaintext, user, scheme, &hash)) {
-		fprintf(stderr, "Unknown scheme: %s\n", scheme);
-		exit(1);
-	}
-	if (Vflag == 1) {
+	if (!password_generate_encoded(plaintext, user, scheme, &hash))
+		i_fatal("Unknown scheme: %s", scheme);
+	if (reverse_verify) {
 		const unsigned char *raw_password;
 		size_t size;
 		const char *error;
 
-		if (password_decode(hash, scheme, &raw_password, &size,
-				    &error) <= 0) {
-			fprintf(stderr, "reverse decode check failed\n");
-			exit(2);
+		if (test_hash != NULL) {
+			scheme = password_get_scheme(&test_hash);
+			if (scheme == NULL)
+				i_fatal("Missing {scheme} prefix from hash");
+			hash = test_hash;
 		}
 
+		if (password_decode(hash, scheme, &raw_password, &size,
+				    &error) <= 0)
+			i_fatal("reverse decode check failed: %s", error);
+
 		if (password_verify(plaintext, user, scheme,
-				    raw_password, size) != 1) {
-			fprintf(stderr,
-				"reverse password verification check failed\n");
-			exit(2);
+				    raw_password, size, &error) <= 0) {
+			i_fatal("reverse password verification check failed: %s",
+				error);
 		}
 
 		printf("{%s}%s (verified)\n", scheme, hash);
-	} else
+	} else {
 		printf("{%s}%s\n", scheme, hash);
+	}
 
 	password_schemes_deinit();
 	random_deinit();
@@ -118,5 +129,5 @@ static void cmd_pw(int argc, char *argv[])
 
 struct doveadm_cmd doveadm_cmd_pw = {
 	cmd_pw, "pw",
-	"[-l] [-p plaintext] [-r rounds] [-s scheme] [-u user] [-V]"
+	"[-l] [-p plaintext] [-r rounds] [-s scheme] [-t hash] [-u user] [-V]"
 };
