@@ -61,6 +61,7 @@ static const struct setting_define doveadm_setting_defines[] = {
 	DEF(SET_STR, doveadm_password),
 	DEF(SET_STR, doveadm_allowed_commands),
 	DEF(SET_STR, dsync_alt_char),
+	DEF(SET_STR, dsync_remote_cmd),
 
 	{ SET_STRLIST, "plugin", offsetof(struct doveadm_settings, plugin_envs), NULL },
 
@@ -77,6 +78,7 @@ const struct doveadm_settings doveadm_default_settings = {
 	.doveadm_password = "",
 	.doveadm_allowed_commands = "",
 	.dsync_alt_char = "_",
+	.dsync_remote_cmd = "ssh -l%{login} %{host} doveadm dsync-server -u%u -l%{lock_timeout} -n%{namespace}",
 
 	.plugin_envs = ARRAY_INIT
 };
