@@ -55,7 +55,7 @@ who_user_has_ip(const struct who_user *user, const struct ip_addr *ip)
 
 static void who_parse_line(const char *line, struct who_line *line_r)
 {
-	const char *const *args = t_strsplit(line, "\t");
+	const char *const *args = t_strsplit_tab(line);
 	const char *ident = args[0];
 	const char *pid_str = args[1];
 	const char *refcount_str = args[2];
