@@ -1,11 +1,15 @@
 #ifndef DOVEADM_H
 #define DOVEADM_H
 
+#include <sysexits.h>
 #include "doveadm-util.h"
 #include "doveadm-settings.h"
 
 #define USAGE_CMDNAME_FMT "  %-12s"
 
+#define DOVEADM_EX_NOTFOUND EX_NOHOST
+#define DOVEADM_EX_NOTPOSSIBLE EX_DATAERR
+
 typedef void doveadm_command_t(int argc, char *argv[]);
 
 struct doveadm_cmd {
@@ -14,6 +18,8 @@ struct doveadm_cmd {
 	const char *short_usage;
 };
 
+extern int doveadm_exit_code;
+
 extern struct doveadm_cmd doveadm_cmd_stop;
 extern struct doveadm_cmd doveadm_cmd_reload;
 extern struct doveadm_cmd doveadm_cmd_auth;
@@ -26,6 +32,8 @@ extern struct doveadm_cmd doveadm_cmd_kick;
 extern struct doveadm_cmd doveadm_cmd_mailbox_mutf7;
 extern struct doveadm_cmd doveadm_cmd_sis_deduplicate;
 extern struct doveadm_cmd doveadm_cmd_sis_find;
+extern struct doveadm_cmd doveadm_cmd_stats_dump;
+extern struct doveadm_cmd doveadm_cmd_stats_top;
 
 void doveadm_register_cmd(const struct doveadm_cmd *cmd);
 
@@ -36,5 +44,7 @@ void doveadm_master_send_signal(int signo);
 void doveadm_register_director_commands(void);
 void doveadm_register_proxy_commands(void);
 void doveadm_register_log_commands(void);
+void doveadm_register_instance_commands(void);
+void doveadm_register_mount_commands(void);
 
 #endif
