@@ -55,12 +55,16 @@ struct service_settings imap_login_service_settings = {
 
 static const struct setting_define imap_login_setting_defines[] = {
 	DEF(SET_STR, imap_capability),
+	DEF(SET_STR, imap_id_send),
+	DEF(SET_STR, imap_id_log),
 
 	SETTING_DEFINE_LIST_END
 };
 
 static const struct imap_login_settings imap_login_default_settings = {
-	.imap_capability = ""
+	.imap_capability = "",
+	.imap_id_send = "",
+	.imap_id_log = ""
 };
 
 static const struct setting_parser_info *imap_login_setting_dependencies[] = {
