@@ -1,16 +1,14 @@
-/* Copyright (c) 2002-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
 
 #include "imap-common.h"
-#include "imap-resp-code.h"
 #include "imap-commands.h"
 
 bool cmd_delete(struct client_command_context *cmd)
 {
 	struct client *client = cmd->client;
-	enum mailbox_name_status status;
 	struct mail_namespace *ns;
 	struct mailbox *box;
-	const char *name, *storage_name;
+	const char *name;
 
 	/* <mailbox> */
 	if (!client_read_string_args(cmd, 1, &name))
@@ -22,23 +20,11 @@ bool cmd_delete(struct client_command_context *cmd)
 		return TRUE;
 	}
 
-	ns = client_find_namespace(cmd, name, &storage_name, &status);
+	ns = client_find_namespace(cmd, &name);
 	if (ns == NULL)
 		return TRUE;
-	switch (status) {
-	case MAILBOX_NAME_EXISTS_MAILBOX:
-	case MAILBOX_NAME_EXISTS_DIR:
-		break;
-	case MAILBOX_NAME_VALID:
-	case MAILBOX_NAME_INVALID:
-	case MAILBOX_NAME_NOINFERIORS:
-		client_fail_mailbox_name_status(cmd, name,
-						IMAP_RESP_CODE_NONEXISTENT,
-						status);
-		return TRUE;
-	}
 
-	box = mailbox_alloc(ns->list, storage_name, 0);
+	box = mailbox_alloc(ns->list, name, 0);
 	if (client->mailbox != NULL &&
 	    mailbox_backends_equal(box, client->mailbox)) {
 		/* deleting selected mailbox. close it first */
