@@ -35,7 +35,6 @@ struct imap_fetch_context {
 	struct mailbox_transaction_context *trans;
 	struct mail_search_args *search_args;
 	struct mail_search_context *search_ctx;
-	struct mail *mail;
 
 	enum mail_fetch_field fetch_data;
 	ARRAY_TYPE(const_string) all_headers;
@@ -65,7 +64,6 @@ struct imap_fetch_context {
 	unsigned int seen_flags_changed:1;
 	unsigned int flags_show_only_seen_changes:1;
 	unsigned int update_partial:1;
-	unsigned int cur_have_eoh:1;
 	unsigned int cur_append_eoh:1;
 	unsigned int first:1;
 	unsigned int line_partial:1;
