@@ -12,7 +12,8 @@ enum search_return_options {
 	SEARCH_RETURN_MODSEQ		= 0x0020,
 	SEARCH_RETURN_SAVE		= 0x0040,
 	SEARCH_RETURN_UPDATE		= 0x0080,
-	SEARCH_RETURN_PARTIAL		= 0x0100
+	SEARCH_RETURN_PARTIAL		= 0x0100,
+	SEARCH_RETURN_RELEVANCY		= 0x0200
 /* Options that don't return any seq/uid results */
 #define SEARCH_RETURN_NORESULTS \
 	(SEARCH_RETURN_ESEARCH | SEARCH_RETURN_MODSEQ | SEARCH_RETURN_SAVE | \
@@ -24,7 +25,6 @@ struct imap_search_context {
 	struct mailbox *box;
 	struct mailbox_transaction_context *trans;
         struct mail_search_context *search_ctx;
-	struct mail *mail;
 
 	struct mail_search_args *sargs;
 	enum search_return_options return_options;
@@ -34,6 +34,9 @@ struct imap_search_context {
 	ARRAY_TYPE(seq_range) result;
 	unsigned int result_count;
 
+	ARRAY_DEFINE(relevancy_scores, float);
+	float min_relevancy, max_relevancy;
+
 	uint64_t highest_seen_modseq;
 	struct timeval start_time;
 
