@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
 
 #include "imap-common.h"
 #include "hex-binary.h"
@@ -11,7 +11,8 @@ int imap_status_parse_items(struct client_command_context *cmd,
 			    struct imap_status_items *items_r)
 {
 	const char *item;
-	enum mailbox_status_items items;
+	enum mailbox_status_items status = 0;
+	enum mailbox_metadata_items metadata = 0;
 
 	if (IMAP_ARG_IS_EOL(args)) {
 		client_send_command_error(cmd, "Empty status list.");
@@ -19,7 +20,6 @@ int imap_status_parse_items(struct client_command_context *cmd,
 	}
 
 	memset(items_r, 0, sizeof(*items_r));
-	items = 0;
 	for (; !IMAP_ARG_IS_EOL(args); args++) {
 		if (!imap_arg_get_atom(args, &item)) {
 			/* list may contain only atoms */
@@ -30,21 +30,21 @@ int imap_status_parse_items(struct client_command_context *cmd,
 
 		item = t_str_ucase(item);
 		if (strcmp(item, "MESSAGES") == 0)
-			items |= STATUS_MESSAGES;
+			status |= STATUS_MESSAGES;
 		else if (strcmp(item, "RECENT") == 0)
-			items |= STATUS_RECENT;
+			status |= STATUS_RECENT;
 		else if (strcmp(item, "UIDNEXT") == 0)
-			items |= STATUS_UIDNEXT;
+			status |= STATUS_UIDNEXT;
 		else if (strcmp(item, "UIDVALIDITY") == 0)
-			items |= STATUS_UIDVALIDITY;
+			status |= STATUS_UIDVALIDITY;
 		else if (strcmp(item, "UNSEEN") == 0)
-			items |= STATUS_UNSEEN;
+			status |= STATUS_UNSEEN;
 		else if (strcmp(item, "HIGHESTMODSEQ") == 0)
-			items |= STATUS_HIGHESTMODSEQ;
+			status |= STATUS_HIGHESTMODSEQ;
 		else if (strcmp(item, "X-SIZE") == 0)
-			items |= STATUS_VIRTUAL_SIZE;
+			metadata |= MAILBOX_METADATA_VIRTUAL_SIZE;
 		else if (strcmp(item, "X-GUID") == 0)
-			items_r->guid = TRUE;
+			metadata |= MAILBOX_METADATA_GUID;
 		else {
 			client_send_tagline(cmd, t_strconcat(
 				"BAD Invalid status item ", item, NULL));
@@ -52,7 +52,8 @@ int imap_status_parse_items(struct client_command_context *cmd,
 		}
 	}
 
-	items_r->mailbox_items = items;
+	items_r->status = status;
+	items_r->metadata = metadata;
 	return 0;
 }
 
@@ -64,7 +65,7 @@ int imap_status_get(struct client_command_context *cmd,
 	struct client *client = cmd->client;
 	struct mailbox *box;
 	enum mail_error error;
-	int ret;
+	int ret = 0;
 
 	if (client->mailbox != NULL &&
 	    mailbox_equals(client->mailbox, ns, mailbox)) {
@@ -72,28 +73,22 @@ int imap_status_get(struct client_command_context *cmd,
 		box = client->mailbox;
 	} else {
 		/* open the mailbox */
-		box = mailbox_alloc(ns->list, mailbox,
-				    MAILBOX_FLAG_READONLY |
-				    MAILBOX_FLAG_KEEP_RECENT);
+		box = mailbox_alloc(ns->list, mailbox, MAILBOX_FLAG_READONLY);
 		if (client->enabled_features != 0)
-			mailbox_enable(box, client->enabled_features);
+			(void)mailbox_enable(box, client->enabled_features);
 	}
 
-	if ((items->mailbox_items & STATUS_HIGHESTMODSEQ) != 0)
-		client_enable(client, MAILBOX_FEATURE_CONDSTORE);
+	if ((items->status & STATUS_HIGHESTMODSEQ) != 0)
+		(void)client_enable(client, MAILBOX_FEATURE_CONDSTORE);
 
-	ret = box == client->mailbox ? 0 : mailbox_sync(box, 0);
-	if (ret == 0) {
-		mailbox_get_status(box, items->mailbox_items,
-				   &result_r->status);
-		if (items->guid)
-			ret = mailbox_get_guid(box, result_r->mailbox_guid);
+	ret = mailbox_get_status(box, items->status, &result_r->status);
+	if (items->metadata != 0 && ret == 0) {
+		ret = mailbox_get_metadata(box, items->metadata,
+					   &result_r->metadata);
 	}
 
 	if (ret < 0) {
-		struct mail_storage *storage = mailbox_get_storage(box);
-
-		*error_r = mail_storage_get_last_error(storage, &error);
+		*error_r = mailbox_get_last_error(box, &error);
 		*error_r = imap_get_error_string(cmd, *error_r, error);
 	}
 	if (box != client->mailbox)
@@ -115,27 +110,27 @@ void imap_status_send(struct client *client, const char *mailbox,
 	str_append(str, " (");
 
 	prefix_len = str_len(str);
-	if ((items->mailbox_items & STATUS_MESSAGES) != 0)
+	if ((items->status & STATUS_MESSAGES) != 0)
 		str_printfa(str, "MESSAGES %u ", status->messages);
-	if ((items->mailbox_items & STATUS_RECENT) != 0)
+	if ((items->status & STATUS_RECENT) != 0)
 		str_printfa(str, "RECENT %u ", status->recent);
-	if ((items->mailbox_items & STATUS_UIDNEXT) != 0)
+	if ((items->status & STATUS_UIDNEXT) != 0)
 		str_printfa(str, "UIDNEXT %u ", status->uidnext);
-	if ((items->mailbox_items & STATUS_UIDVALIDITY) != 0)
+	if ((items->status & STATUS_UIDVALIDITY) != 0)
 		str_printfa(str, "UIDVALIDITY %u ", status->uidvalidity);
-	if ((items->mailbox_items & STATUS_UNSEEN) != 0)
+	if ((items->status & STATUS_UNSEEN) != 0)
 		str_printfa(str, "UNSEEN %u ", status->unseen);
-	if ((items->mailbox_items & STATUS_HIGHESTMODSEQ) != 0) {
+	if ((items->status & STATUS_HIGHESTMODSEQ) != 0) {
 		str_printfa(str, "HIGHESTMODSEQ %llu ",
 			    (unsigned long long)status->highest_modseq);
 	}
-	if ((items->mailbox_items & STATUS_VIRTUAL_SIZE) != 0) {
+	if ((items->metadata & MAILBOX_METADATA_VIRTUAL_SIZE) != 0) {
 		str_printfa(str, "X-SIZE %llu ",
-			    (unsigned long long)status->virtual_size);
+			    (unsigned long long)result->metadata.virtual_size);
 	}
-	if (items->guid) {
+	if ((items->metadata & MAILBOX_METADATA_GUID) != 0) {
 		str_printfa(str, "X-GUID %s ",
-			    mail_guid_128_to_string(result->mailbox_guid));
+			    guid_128_to_string(result->metadata.guid));
 	}
 
 	if (str_len(str) != prefix_len)
