@@ -127,7 +127,7 @@ static void ipc_connection_input(struct ipc_connection *conn)
 		if ((line = i_stream_next_line(conn->input)) == NULL)
 			return;
 
-		args = t_strsplit(line, "\t");
+		args = t_strsplit_tab(line);
 		if (str_array_length(args) < 3 ||
 		    strcmp(args[0], "HANDSHAKE") != 0) {
 			i_error("IPC server sent invalid handshake");
