@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2005-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -37,9 +37,9 @@ void auth_client_connect(struct auth_client *client)
 		auth_server_connection_connect(client->conn);
 }
 
-void auth_client_disconnect(struct auth_client *client)
+void auth_client_disconnect(struct auth_client *client, const char *reason)
 {
-	auth_server_connection_disconnect(client->conn);
+	auth_server_connection_disconnect(client->conn, reason);
 }
 
 bool auth_client_is_connected(struct auth_client *client)
