@@ -1,49 +1,26 @@
-/* Copyright (c) 2010-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "istream.h"
-#include "ostream-internal.h"
+#include "ostream-private.h"
 #include "ostream-cmp.h"
 
 struct cmp_ostream {
 	struct ostream_private ostream;
 
 	struct istream *input;
-	struct ostream *output;
 	bool equals;
 };
 
-static void cstream_copy_error(struct cmp_ostream *cstream)
-{
-	struct ostream *src = cstream->output;
-	struct ostream *dest = &cstream->ostream.ostream;
-
-	dest->stream_errno = src->stream_errno;
-	dest->last_failed_errno = src->last_failed_errno;
-	dest->overflow = src->overflow;
-}
-
 static void o_stream_cmp_close(struct iostream_private *stream)
 {
 	struct cmp_ostream *cstream = (struct cmp_ostream *)stream;
 
-	if (cstream->output == NULL)
+	if (cstream->input == NULL)
 		return;
 
 	i_stream_unref(&cstream->input);
 	o_stream_flush(&cstream->ostream.ostream);
-	o_stream_unref(&cstream->output);
-}
-
-static int o_stream_cmp_flush(struct ostream_private *stream)
-{
-	struct cmp_ostream *cstream = (struct cmp_ostream *)stream;
-	int ret;
-
-	ret = o_stream_flush(cstream->output);
-	if (ret < 0)
-		cstream_copy_error(cstream);
-	return ret;
 }
 
 bool stream_cmp_block(struct istream *input,
@@ -82,8 +59,8 @@ o_stream_cmp_sendv(struct ostream_private *stream,
 		}
 	}
 
-	if ((ret = o_stream_sendv(cstream->output, iov, iov_count)) < 0) {
-		cstream_copy_error(cstream);
+	if ((ret = o_stream_sendv(stream->parent, iov, iov_count)) < 0) {
+		o_stream_copy_error_from_parent(stream);
 		return -1;
 	}
 
@@ -98,15 +75,12 @@ o_stream_create_cmp(struct ostream *output, struct istream *input)
 
 	cstream = i_new(struct cmp_ostream, 1);
 	cstream->ostream.sendv = o_stream_cmp_sendv;
-	cstream->ostream.flush = o_stream_cmp_flush;
 	cstream->ostream.iostream.close = o_stream_cmp_close;
 	cstream->input = input;
-	cstream->output = output;
 	cstream->equals = TRUE;
 	i_stream_ref(input);
-	o_stream_ref(output);
 
-	return o_stream_create(&cstream->ostream);
+	return o_stream_create(&cstream->ostream, output);
 }
 
 bool o_stream_cmp_equals(struct ostream *_output)
