@@ -10,7 +10,8 @@ void imap_bodystructure_parse_header(pool_t pool, struct message_part *part,
 
 /* Returns TRUE if BODYSTRUCTURE is
    ("text" "plain" ("charset" "us-ascii") NIL NIL "7bit" n n NIL NIL NIL) */
-bool imap_bodystructure_is_plain_7bit(struct message_part *part);
+bool imap_bodystructure_is_plain_7bit(const struct message_part *part)
+	ATTR_PURE;
 
 void imap_bodystructure_write(const struct message_part *part,
 			      string_t *dest, bool extended);
