@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "istream.h"
@@ -22,6 +22,7 @@ enum arg_parse_type {
 
 struct imap_parser {
 	/* permanent */
+	int refcount;
 	pool_t pool;
 	struct istream *input;
 	struct ostream *output;
@@ -56,6 +57,7 @@ imap_parser_create(struct istream *input, struct ostream *output,
 	struct imap_parser *parser;
 
 	parser = i_new(struct imap_parser, 1);
+	parser->refcount = 1;
 	parser->pool = pool_alloconly_create(MEMPOOL_GROWING"IMAP parser",
 					     1024*10);
 	parser->input = input;
@@ -67,8 +69,20 @@ imap_parser_create(struct istream *input, struct ostream *output,
 	return parser;
 }
 
-void imap_parser_destroy(struct imap_parser **parser)
+void imap_parser_ref(struct imap_parser *parser)
 {
+	i_assert(parser->refcount > 0);
+
+	parser->refcount++;
+}
+
+void imap_parser_unref(struct imap_parser **parser)
+{
+	i_assert((*parser)->refcount > 0);
+
+	if (--(*parser)->refcount > 0)
+		return;
+
 	pool_unref(&(*parser)->pool);
 	i_free(*parser);
 	*parser = NULL;
@@ -175,6 +189,17 @@ static int imap_parser_close_list(struct imap_parser *parser)
 	return TRUE;
 }
 
+static char *
+imap_parser_strdup(struct imap_parser *parser,
+		   const void *data, size_t len)
+{
+	char *ret;
+
+	ret = p_malloc(parser->pool, len + 1);
+	memcpy(ret, data, len);
+	return ret;
+}
+
 static void imap_parser_save_arg(struct imap_parser *parser,
 				 const unsigned char *data, size_t size)
 {
@@ -191,7 +216,8 @@ static void imap_parser_save_arg(struct imap_parser *parser,
 		} else {
 			/* simply save the string */
 			arg->type = IMAP_ARG_ATOM;
-			arg->_data.str = p_strndup(parser->pool, data, size);
+			arg->_data.str = imap_parser_strdup(parser, data, size);
+			arg->str_len = size;
 		}
 		break;
 	case ARG_PARSE_STRING:
@@ -208,6 +234,7 @@ static void imap_parser_save_arg(struct imap_parser *parser,
 			str_unescape(str + parser->str_first_escape-1);
 		}
 		arg->_data.str = str;
+		arg->str_len = strlen(str);
 		break;
 	case ARG_PARSE_LITERAL_DATA:
 		if ((parser->flags & IMAP_PARSE_FLAG_LITERAL_SIZE) != 0) {
@@ -220,14 +247,12 @@ static void imap_parser_save_arg(struct imap_parser *parser,
 		}
 		/* fall through */
 	case ARG_PARSE_LITERAL_DATA_FORCED:
-		if ((parser->flags &
-			    IMAP_PARSE_FLAG_LITERAL_TYPE) != 0) {
+		if ((parser->flags & IMAP_PARSE_FLAG_LITERAL_TYPE) != 0)
 			arg->type = IMAP_ARG_LITERAL;
-			arg->_data.str = p_strndup(parser->pool, data, size);
-		} else {
+		else
 			arg->type = IMAP_ARG_STRING;
-			arg->_data.str = p_strndup(parser->pool, data, size);
-		}
+		arg->_data.str = imap_parser_strdup(parser, data, size);
+		arg->str_len = size;
 		break;
 	default:
                 i_unreached();
