@@ -1,4 +1,4 @@
-/* Copyright (c) 2008-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2008-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "str.h"
@@ -55,6 +55,17 @@ mbase64_encode(string_t *dest, const unsigned char *in, unsigned int len)
 	str_append_c(dest, '-');
 }
 
+static const char *imap_utf8_first_encode_char(const char *str)
+{
+	const char *p;
+
+	for (p = str; *p != '\0'; p++) {
+		if (*p == '&' || (unsigned char)*p >= 0x80)
+			return p;
+	}
+	return NULL;
+}
+
 int imap_utf8_to_utf7(const char *src, string_t *dest)
 {
 	const char *p;
@@ -62,12 +73,9 @@ int imap_utf8_to_utf7(const char *src, string_t *dest)
 	uint8_t *utf16, *u;
 	uint16_t u16;
 
-	for (p = src; *p != '\0'; p++) {
-		if (*p == '&' || (unsigned char)*p >= 0x80)
-			break;
-	}
-	if (*p == '\0') {
-		/* no ASCII characters that need to be encoded */
+	p = imap_utf8_first_encode_char(src);
+	if (p == NULL) {
+		/* no characters that need to be encoded */
 		str_append(dest, src);
 		return 0;
 	}
@@ -110,6 +118,22 @@ int imap_utf8_to_utf7(const char *src, string_t *dest)
 	return 0;
 }
 
+int t_imap_utf8_to_utf7(const char *src, const char **dest_r)
+{
+	string_t *str;
+	int ret;
+
+	if (imap_utf8_first_encode_char(src) == NULL) {
+		*dest_r = src;
+		return 0;
+	}
+
+	str = t_str_new(64);
+	ret = imap_utf8_to_utf7(src, str);
+	*dest_r = str_c(str);
+	return ret;
+}
+
 static int utf16buf_to_utf8(string_t *dest, const unsigned char output[4],
 			    unsigned int *_pos, unsigned int len)
 {
