@@ -88,6 +88,11 @@ void mail_index_modseq_enable(struct mail_index *index)
 	index->modseqs_enabled = TRUE;
 }
 
+bool mail_index_have_modseq_tracking(struct mail_index *index)
+{
+	return mail_index_map_get_modseq_header(index->map) != NULL;
+}
+
 const struct mail_index_modseq_header *
 mail_index_map_get_modseq_header(struct mail_index_map *map)
 {
