@@ -1,4 +1,4 @@
-/* Copyright (c) 2003-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2003-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -77,6 +77,7 @@ void mail_index_free(struct mail_index **_index)
 	array_free(&index->keywords);
 	array_free(&index->module_contexts);
 
+	i_free(index->ext_hdr_init_data);
 	i_free(index->gid_origin);
 	i_free(index->error);
 	i_free(index->dir);
@@ -110,6 +111,23 @@ void mail_index_set_lock_method(struct mail_index *index,
 	index->max_lock_timeout_secs = max_timeout_secs;
 }
 
+void mail_index_set_ext_init_data(struct mail_index *index, uint32_t ext_id,
+				  const void *data, size_t size)
+{
+	const struct mail_index_registered_ext *rext;
+
+	i_assert(index->ext_hdr_init_data == NULL ||
+		 index->ext_hdr_init_id == ext_id);
+
+	rext = array_idx(&index->extensions, ext_id);
+	i_assert(rext->hdr_size == size);
+
+	index->ext_hdr_init_id = ext_id;
+	i_free(index->ext_hdr_init_data);
+	index->ext_hdr_init_data = i_malloc(size);
+	memcpy(index->ext_hdr_init_data, data, size);
+}
+
 uint32_t mail_index_ext_register(struct mail_index *index, const char *name,
 				 uint32_t default_hdr_size,
 				 uint16_t default_record_size,
@@ -504,8 +522,33 @@ static int mail_index_open_files(struct mail_index *index,
 			return -1;
 	}
 
-	index->cache = created ? mail_cache_create(index) :
-		mail_cache_open_or_create(index);
+	if (index->cache == NULL) {
+		index->cache = created ? mail_cache_create(index) :
+			mail_cache_open_or_create(index);
+	}
+	return 1;
+}
+
+static int
+mail_index_open_opened(struct mail_index *index,
+		       enum mail_index_open_flags flags)
+{
+	int ret;
+
+	i_assert(index->map != NULL);
+
+	if ((index->map->hdr.flags & MAIL_INDEX_HDR_FLAG_CORRUPTED) != 0) {
+		/* index was marked corrupted. we'll probably need to
+		   recreate the files. */
+		if (index->map != NULL)
+			mail_index_unmap(&index->map);
+		mail_index_close_file(index);
+		mail_transaction_log_close(index->log);
+		if ((ret = mail_index_open_files(index, flags)) <= 0)
+			return ret;
+	}
+
+	index->open_count++;
 	return 1;
 }
 
@@ -514,9 +557,12 @@ int mail_index_open(struct mail_index *index, enum mail_index_open_flags flags)
 	int ret;
 
 	if (index->open_count > 0) {
-		i_assert(index->map != NULL);
-		index->open_count++;
-		return 1;
+		if ((ret = mail_index_open_opened(index, flags)) <= 0) {
+			/* doesn't exist and create flag not used */
+			index->open_count++;
+			mail_index_close(index);
+		}
+		return ret;
 	}
 
 	index->filepath = MAIL_INDEX_IS_IN_MEMORY(index) ?
@@ -542,6 +588,9 @@ int mail_index_open(struct mail_index *index, enum mail_index_open_flags flags)
 	    (flags & MAIL_INDEX_OPEN_FLAG_MMAP_DISABLE) == 0)
 		i_fatal("nfs flush requires mmap_disable=yes");
 
+	/* NOTE: increase open_count only after mail_index_open_files().
+	   it's used elsewhere to check if we're doing an initial opening
+	   of the index files */
 	if ((ret = mail_index_open_files(index, flags)) <= 0) {
 		/* doesn't exist and create flag not used */
 		index->open_count++;
@@ -589,6 +638,9 @@ void mail_index_close(struct mail_index *index)
 	if (--index->open_count > 0)
 		return;
 
+	i_assert(!index->syncing);
+	i_assert(index->view_count == 0);
+
 	if (index->map != NULL)
 		mail_index_unmap(&index->map);
 
@@ -768,6 +820,7 @@ void mail_index_mark_corrupted(struct mail_index *index)
 		if (unlink(index->filepath) < 0 &&
 		    errno != ENOENT && errno != ESTALE)
 			mail_index_set_syscall_error(index, "unlink()");
+		(void)mail_transaction_log_unlink(index->log);
 	}
 }
 
