@@ -1,15 +1,26 @@
 #ifndef MESSAGE_DECODER_H
 #define MESSAGE_DECODER_H
 
+enum message_decoder_flags {
+	/* Return all headers and parts through
+	   uni_utf8_to_decomposed_titlecase() */
+	MESSAGE_DECODER_FLAG_DTCASE		= 0x01,
+	/* Return binary MIME parts as-is without any conversion. */
+	MESSAGE_DECODER_FLAG_RETURN_BINARY	= 0x02
+};
+
 struct message_block;
 
 /* Decode message's contents as UTF-8, both the headers and the MIME bodies.
-   The bodies are decoded from quoted-printable and base64 formats if needed.
-   If dtcase=TRUE, the data is returned through
-   uni_utf8_to_decomposed_titlecase(). */
-struct message_decoder_context *message_decoder_init(bool dtcase);
+   The bodies are decoded from quoted-printable and base64 formats if needed. */
+struct message_decoder_context *
+message_decoder_init(enum message_decoder_flags flags);
 void message_decoder_deinit(struct message_decoder_context **ctx);
 
+/* Change the MESSAGE_DECODER_FLAG_RETURN_BINARY flag */
+void message_decoder_set_return_binary(struct message_decoder_context *ctx,
+				       bool set);
+
 /* Decode input and return decoded output. Headers are returned only in their
    full multiline forms.
 
