@@ -7,14 +7,16 @@ struct message_search_context;
 
 enum message_search_flags {
 	/* Skip the main header and all the MIME headers. */
-	MESSAGE_SEARCH_FLAG_SKIP_HEADERS	= 0x01
+	MESSAGE_SEARCH_FLAG_SKIP_HEADERS	= 0x01,
+	/* Search with decomposed titlecase (instead of exact case matching).
+	   The search key must be given with dtcase also. */
+	MESSAGE_SEARCH_FLAG_DTCASE		= 0x02
 };
 
-/* Returns 1 if ok, 0 if unknown charset, -1 if key contains invalid characters
-   in given charset. */
-int message_search_init(const char *key, const char *charset,
-			enum message_search_flags flags,
-			struct message_search_context **ctx_r);
+/* The key must be given in UTF-8 charset */
+struct message_search_context *
+message_search_init(const char *key_utf8,
+		    enum message_search_flags flags);
 void message_search_deinit(struct message_search_context **ctx);
 
 /* Returns TRUE if key is found from input buffer, FALSE if not. */
@@ -27,6 +29,6 @@ void message_search_reset(struct message_search_context *ctx);
 /* Search a full message. Returns 1 if match was found, 0 if not,
    -1 if error (if stream_error == 0, the parts contained broken data) */
 int message_search_msg(struct message_search_context *ctx,
-		       struct istream *input, const struct message_part *parts);
+		       struct istream *input, struct message_part *parts);
 
 #endif
