@@ -22,8 +22,10 @@ static int cydir_mail_stat(struct mail *mail, struct stat *st_r)
 {
 	const char *path;
 
-	if (mail->lookup_abort == MAIL_LOOKUP_ABORT_NOT_IN_CACHE)
-		return mail_set_aborted(mail);
+	if (mail->lookup_abort == MAIL_LOOKUP_ABORT_NOT_IN_CACHE) {
+		mail_set_aborted(mail);
+		return -1;
+	}
 
 	mail->transaction->stats.stat_lookup_count++;
 	path = cydir_mail_get_path(mail);
