@@ -407,7 +407,7 @@ dbox_file_seek_next_at_metadata(struct dbox_file *file, uoff_t *offset)
 
 	/* skip over the actual metadata */
 	buf_size = i_stream_get_max_buffer_size(file->input);
-	i_stream_set_max_buffer_size(file->input, 0);
+	i_stream_set_max_buffer_size(file->input, (size_t)-1);
 	while ((line = i_stream_read_next_line(file->input)) != NULL) {
 		if (*line == DBOX_METADATA_OLDV1_SPACE || *line == '\0') {
 			/* end of metadata */
@@ -588,7 +588,7 @@ int dbox_file_get_append_stream(struct dbox_file_append_context *ctx,
 		return 0;
 	}
 
-	if (ctx->output->offset == 0 && !file->created) {
+	if (ctx->output->offset == 0) {
 		/* first append to existing file. seek to eof first. */
 		if (fstat(file->fd, &st) < 0) {
 			dbox_file_set_syscall_error(file, "fstat()");
@@ -657,7 +657,8 @@ dbox_file_metadata_read_at(struct dbox_file *file, uoff_t metadata_offset)
 
 	ret = 0;
 	buf_size = i_stream_get_max_buffer_size(file->input);
-	i_stream_set_max_buffer_size(file->input, 0);
+	/* use unlimited line length for metadata */
+	i_stream_set_max_buffer_size(file->input, (size_t)-1);
 	while ((line = i_stream_read_next_line(file->input)) != NULL) {
 		if (*line == DBOX_METADATA_OLDV1_SPACE || *line == '\0') {
 			/* end of metadata */
