@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2007-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
@@ -29,7 +29,7 @@ int mdbox_mail_lookup(struct mdbox_mailbox *mbox, struct mail_index_view *view,
 		mail_index_lookup_uid(view, seq, &uid);
 		mail_storage_set_critical(&mbox->storage->storage.storage,
 			"mdbox %s: map uid lost for uid %u",
-			mbox->box.path, uid);
+			mailbox_get_path(&mbox->box), uid);
 		mdbox_storage_set_corrupted(mbox->storage);
 		return -1;
 	}
@@ -46,7 +46,7 @@ int mdbox_mail_lookup(struct mdbox_mailbox *mbox, struct mail_index_view *view,
 	if (cur_map_uid_validity != mbox->map_uid_validity) {
 		mail_storage_set_critical(&mbox->storage->storage.storage,
 			"mdbox %s: map uidvalidity mismatch (%u vs %u)",
-			mbox->box.path, mbox->map_uid_validity,
+			mailbox_get_path(&mbox->box), mbox->map_uid_validity,
 			cur_map_uid_validity);
 		mdbox_storage_set_corrupted(mbox->storage);
 		return -1;
@@ -126,7 +126,7 @@ int mdbox_mail_open(struct dbox_mail *mail, uoff_t *offset_r,
 		}
 
 		if (!dbox_file_is_open(mail->open_file))
-			mail->imail.mail.stats_open_lookup_count++;
+			_mail->transaction->stats.open_lookup_count++;
 		if (dbox_file_open(mail->open_file, &deleted) <= 0)
 			return -1;
 		if (deleted) {
@@ -190,6 +190,9 @@ struct mail_vfuncs mdbox_mail_vfuncs = {
 	index_mail_set_seq,
 	index_mail_set_uid,
 	index_mail_set_uid_cache_updates,
+	index_mail_prefetch,
+	index_mail_precache,
+	index_mail_add_temp_wanted_fields,
 
 	index_mail_get_flags,
 	index_mail_get_keywords,
@@ -212,7 +215,6 @@ struct mail_vfuncs mdbox_mail_vfuncs = {
 	index_mail_update_modseq,
 	NULL,
 	index_mail_expunge,
-	index_mail_parse,
 	index_mail_set_cache_corrupted,
 	index_mail_opened
 };
