@@ -1,4 +1,4 @@
-/* Copyright (c) 2010-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "safe-mkstemp.h"
@@ -147,7 +147,6 @@ static void index_attachment_save_mail_header(struct mail_save_context *ctx,
 static bool save_is_attachment(struct mail_save_context *ctx,
 			       struct message_part *part)
 {
-	struct mailbox *box = ctx->transaction->box;
 	struct mail_attachment_part apart;
 
 	if ((part->flags & MESSAGE_PART_FLAG_MULTIPART) != 0) {
@@ -155,14 +154,14 @@ static bool save_is_attachment(struct mail_save_context *ctx,
 		   but they're never themselves */
 		return FALSE;
 	}
-	if (box->v.save_is_attachment == NULL)
+	if (ctx->part_is_attachment == NULL)
 		return TRUE;
 
 	memset(&apart, 0, sizeof(apart));
 	apart.part = part;
 	apart.content_type = ctx->attach->part.content_type;
 	apart.content_disposition = ctx->attach->part.content_disposition;
-	return box->v.save_is_attachment(ctx, &apart);
+	return ctx->part_is_attachment(ctx, &apart);
 }
 
 static int index_attachment_save_temp_open_fd(struct mail_storage *storage)
@@ -351,7 +350,7 @@ static int index_attachment_save_finish_part(struct mail_save_context *ctx)
 	struct fs_file *file;
 	struct istream *input;
 	struct ostream *output;
-	uint8_t guid_128[MAIL_GUID_128_SIZE];
+	guid_128_t guid_128;
 	const char *attachment_dir, *path, *digest;
 	string_t *digest_str;
 	const unsigned char *data;
@@ -394,12 +393,12 @@ static int index_attachment_save_finish_part(struct mail_save_context *ctx)
 		digest = t_strconcat(digest, "\0\0\0\0", NULL);
 	}
 
-	mail_generate_guid_128(guid_128);
+	guid_128_generate(guid_128);
 	attachment_dir = index_attachment_dir_get(storage);
 	path = t_strdup_printf("%s/%c%c/%c%c/%s-%s", attachment_dir,
 			       digest[0], digest[1],
 			       digest[2], digest[3], digest,
-			       mail_guid_128_to_string(guid_128));
+			       guid_128_to_string(guid_128));
 	if (fs_open(ctx->attach->fs, path,
 		    FS_OPEN_MODE_CREATE | flags, &file) < 0) {
 		mail_storage_set_critical(storage, "%s",
