@@ -519,10 +519,13 @@ int index_storage_mailbox_delete(struct mailbox *box)
 		return index_storage_mailbox_delete_dir(box, FALSE);
 	}
 
-	/* specifically support symlinked shared mailboxes. a deletion will
-	   simply remove the symlink, not actually expunge any mails */
-	if (mailbox_list_delete_symlink(box->list, box->name) == 0)
-		return 0;
+	if ((box->list->flags & MAILBOX_LIST_FLAG_MAILBOX_FILES) == 0) {
+		/* specifically support symlinked shared mailboxes. a deletion
+		   will simply remove the symlink, not actually expunge any
+		   mails */
+		if (mailbox_list_delete_symlink(box->list, box->name) == 0)
+			return 0;
+	}
 
 	/* we can't easily atomically delete all mails and the mailbox. so:
 	   1) expunge all mails
@@ -532,8 +535,10 @@ int index_storage_mailbox_delete(struct mailbox *box)
 	     no) finish deleting the mailbox
 	*/
 
-	if (mailbox_expunge_all_mails(box) < 0)
-		return -1;
+	if (!box->deleting_must_be_empty) {
+		if (mailbox_expunge_all_mails(box) < 0)
+			return -1;
+	}
 	if (mailbox_mark_index_deleted(box, TRUE) < 0)
 		return -1;
 
