@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2007-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
@@ -71,7 +71,7 @@ void index_sync_changes_delete_to(struct index_sync_changes_context *ctx,
 static bool
 index_sync_changes_have_expunges(struct index_sync_changes_context *ctx,
 				 unsigned int count,
-				 uint8_t expunged_guid_128[MAIL_GUID_128_SIZE])
+				 guid_128_t expunged_guid_128_r)
 {
 	const struct mail_index_sync_rec *syncs;
 	unsigned int i;
@@ -79,8 +79,8 @@ index_sync_changes_have_expunges(struct index_sync_changes_context *ctx,
 	syncs = array_idx(&ctx->syncs, 0);
 	for (i = 0; i < count; i++) {
 		if (syncs[i].type == MAIL_INDEX_SYNC_TYPE_EXPUNGE) {
-			memcpy(expunged_guid_128, syncs[i].guid_128,
-			       MAIL_GUID_128_SIZE);
+			memcpy(expunged_guid_128_r, syncs[i].guid_128,
+			       GUID_128_SIZE);
 			return TRUE;
 		}
 	}
@@ -89,7 +89,7 @@ index_sync_changes_have_expunges(struct index_sync_changes_context *ctx,
 
 void index_sync_changes_read(struct index_sync_changes_context *ctx,
 			     uint32_t uid, bool *sync_expunge_r,
-			     uint8_t expunged_guid_128[MAIL_GUID_128_SIZE])
+			     guid_128_t expunged_guid_128_r)
 {
 	struct mail_index_sync_rec *sync_rec = &ctx->sync_rec;
 	uint32_t seq1, seq2;
@@ -107,8 +107,8 @@ void index_sync_changes_read(struct index_sync_changes_context *ctx,
 
 			if (sync_rec->type == MAIL_INDEX_SYNC_TYPE_EXPUNGE) {
 				*sync_expunge_r = TRUE;
-				memcpy(expunged_guid_128, sync_rec->guid_128,
-				       MAIL_GUID_128_SIZE);
+				memcpy(expunged_guid_128_r, sync_rec->guid_128,
+				       GUID_128_SIZE);
 			}
 		}
 
@@ -151,7 +151,7 @@ void index_sync_changes_read(struct index_sync_changes_context *ctx,
 	if (!*sync_expunge_r && orig_count > 0) {
 		*sync_expunge_r =
 			index_sync_changes_have_expunges(ctx, orig_count,
-							 expunged_guid_128);
+							 expunged_guid_128_r);
 	}
 }
 
