@@ -102,7 +102,7 @@ maildir_open_mail(struct maildir_mailbox *mbox, struct mail *mail,
 	return input;
 }
 
-static int maildir_mail_stat(struct mail *mail, struct stat *st)
+static int maildir_mail_stat(struct mail *mail, struct stat *st_r)
 {
 	struct maildir_mailbox *mbox = (struct maildir_mailbox *)mail->box;
 	struct index_mail *imail = (struct index_mail *)mail;
@@ -110,8 +110,10 @@ static int maildir_mail_stat(struct mail *mail, struct stat *st)
 	const char *path;
 	int ret;
 
-	if (mail->lookup_abort == MAIL_LOOKUP_ABORT_NOT_IN_CACHE)
-		return mail_set_aborted(mail);
+	if (mail->lookup_abort == MAIL_LOOKUP_ABORT_NOT_IN_CACHE) {
+		mail_set_aborted(mail);
+		return -1;
+	}
 
 	if (imail->data.access_part != 0 &&
 	    imail->data.stream == NULL) {
@@ -126,10 +128,10 @@ static int maildir_mail_stat(struct mail *mail, struct stat *st)
 		stp = i_stream_stat(imail->data.stream, FALSE);
 		if (stp == NULL)
 			return -1;
-		*st = *stp;
+		*st_r = *stp;
 	} else if (!mail->saving) {
 		mail->transaction->stats.stat_lookup_count++;
-		ret = maildir_file_do(mbox, mail->uid, do_stat, st);
+		ret = maildir_file_do(mbox, mail->uid, do_stat, st_r);
 		if (ret <= 0) {
 			if (ret == 0)
 				mail_set_expunged(mail);
@@ -138,7 +140,7 @@ static int maildir_mail_stat(struct mail *mail, struct stat *st)
 	} else {
 		mail->transaction->stats.stat_lookup_count++;
 		path = maildir_save_file_get_path(mail->transaction, mail->seq);
-		if (stat(path, st) < 0) {
+		if (stat(path, st_r) < 0) {
 			mail_storage_set_critical(mail->box->storage,
 						  "stat(%s) failed: %m", path);
 			return -1;
@@ -480,6 +482,11 @@ maildir_mail_get_special(struct mail *_mail, enum mail_fetch_field field,
 		/* use GUID from uidlist if it exists */
 		i_assert(!_mail->saving);
 
+		if (mail->data.guid != NULL) {
+			*value_r = mail->data.guid;
+			return 0;
+		}
+
 		/* first make sure that we have a refreshed uidlist */
 		if (maildir_mail_get_fname(mbox, _mail, &fname) <= 0)
 			return -1;
@@ -488,7 +495,8 @@ maildir_mail_get_special(struct mail *_mail, enum mail_fetch_field field,
 						  MAILDIR_UIDLIST_REC_EXT_GUID);
 		if (guid != NULL) {
 			if (*guid != '\0') {
-				*value_r = p_strdup(mail->data_pool, guid);
+				*value_r = mail->data.guid =
+					p_strdup(mail->data_pool, guid);
 				return 0;
 			}
 
@@ -501,9 +509,14 @@ maildir_mail_get_special(struct mail *_mail, enum mail_fetch_field field,
 		}
 
 		/* default to base filename: */
+		if (maildir_mail_get_special(_mail, MAIL_FETCH_UIDL_FILE_NAME,
+					     value_r) < 0)
+			return -1;
+		mail->data.guid = mail->data.filename;
+		return 0;
 	case MAIL_FETCH_UIDL_FILE_NAME:
-		if (mail->data.guid != NULL) {
-			*value_r = mail->data.guid;
+		if (mail->data.filename != NULL) {
+			*value_r = mail->data.filename;
 			return 0;
 		}
 		if (fname != NULL) {
@@ -519,10 +532,10 @@ maildir_mail_get_special(struct mail *_mail, enum mail_fetch_field field,
 			fname = fname != NULL ? fname + 1 : path;
 		}
 		end = strchr(fname, MAILDIR_INFO_SEP);
-		mail->data.guid = end == NULL ?
+		mail->data.filename = end == NULL ?
 			p_strdup(mail->data_pool, fname) :
 			p_strdup_until(mail->data_pool, fname, end);
-		*value_r = mail->data.guid;
+		*value_r = mail->data.filename;
 		return 0;
 	case MAIL_FETCH_UIDL_BACKEND:
 		uidl = maildir_uidlist_lookup_ext(mbox->uidlist, _mail->uid,
