@@ -34,6 +34,7 @@ struct maildir_filename {
 
 	uoff_t size, vsize;
 	enum mail_flags flags;
+	unsigned int pop3_order;
 	unsigned int preserve_filename:1;
 	unsigned int keywords_count;
 	/* unsigned int keywords[]; */
@@ -187,6 +188,7 @@ maildir_save_add(struct mail_save_context *_ctx, const char *tmp_fname,
 	}
 	if (_ctx->pop3_uidl != NULL)
 		mf->pop3_uidl = p_strdup(ctx->pool, _ctx->pop3_uidl);
+	mf->pop3_order = _ctx->pop3_order;
 
 	/* insert into index */
 	mail_index_append(ctx->trans, _ctx->uid, &ctx->seq);
@@ -516,6 +518,7 @@ static int maildir_save_finish_real(struct mail_save_context *_ctx)
 	struct mail_storage *storage = &ctx->mbox->storage->storage;
 	const char *path;
 	off_t real_size;
+	uoff_t size;
 	int output_errno;
 
 	ctx->last_save_finished = TRUE;
@@ -575,10 +578,16 @@ static int maildir_save_finish_real(struct mail_save_context *_ctx)
 	if (real_size == (off_t)-1) {
 		mail_storage_set_critical(storage,
 					  "lseek(%s) failed: %m", path);
-	} else if (real_size != (off_t)ctx->file_last->size) {
+	} else if (real_size != (off_t)ctx->file_last->size &&
+		   (!maildir_filename_get_size(ctx->file_last->dest_basename,
+					       MAILDIR_EXTRA_FILE_SIZE, &size) ||
+		    size != ctx->file_last->size)) {
 		/* e.g. zlib plugin was used. the "physical size" must be in
 		   the maildir filename, since stat() will return wrong size */
 		ctx->file_last->preserve_filename = FALSE;
+		/* reset the base name as well, just in case there's a
+		   ,W=vsize */
+		ctx->file_last->dest_basename = ctx->file_last->tmp_name;
 	}
 	if (close(ctx->fd) < 0) {
 		if (!mail_storage_set_error_from_errno(storage)) {
@@ -922,6 +931,11 @@ static void maildir_save_sync_uidlist(struct maildir_save_context *ctx)
 				MAILDIR_UIDLIST_REC_EXT_POP3_UIDL,
 				mf->pop3_uidl);
 		}
+		if (mf->pop3_order > 0) {
+			maildir_uidlist_sync_set_ext(ctx->uidlist_sync_ctx, rec,
+				MAILDIR_UIDLIST_REC_EXT_POP3_ORDER,
+				t_strdup_printf("%u", mf->pop3_order));
+		}
 	} T_END;
 	i_assert(!seq_range_array_iter_nth(&iter, n, &uid));
 }
