@@ -1,8 +1,8 @@
-/* Copyright (c) 2003-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2003-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "buffer.h"
-#include "istream-internal.h"
+#include "istream-private.h"
 #include "istream-raw-mbox.h"
 #include "mbox-from.h"
 
