@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "eacces-error.h"
@@ -174,7 +174,7 @@ static int mbox_file_open_latest(struct mbox_lock_context *ctx, int lock_type)
 		   be sure that the file is latest, but mbox files get rarely
 		   deleted and the flushing might cause errors (e.g. EBUSY for
 		   trying to flush a /var/mail mountpoint) */
-		if (nfs_safe_stat(mbox->box.path, &st) < 0) {
+		if (nfs_safe_stat(mailbox_get_path(&mbox->box), &st) < 0) {
 			if (errno == ENOENT)
 				mailbox_set_deleted(&mbox->box);
 			else
@@ -250,7 +250,7 @@ static int mbox_dotlock_privileged_op(struct mbox_mailbox *mbox,
 				      struct dotlock_settings *set,
 				      enum mbox_dotlock_op op)
 {
-	const char *dir, *fname;
+	const char *box_path, *dir, *fname;
 	int ret = -1, orig_dir_fd, orig_errno;
 
 	orig_dir_fd = open(".", O_RDONLY);
@@ -268,12 +268,13 @@ static int mbox_dotlock_privileged_op(struct mbox_mailbox *mbox,
 	      privileged group
 	    - DoS other users by dotlocking their mailboxes infinitely
 	*/
-	fname = strrchr(mbox->box.path, '/');
+	box_path = mailbox_get_path(&mbox->box);
+	fname = strrchr(box_path, '/');
 	if (fname == NULL) {
 		/* already relative */
-		fname = mbox->box.path;
+		fname = box_path;
 	} else {
-		dir = t_strdup_until(mbox->box.path, fname);
+		dir = t_strdup_until(box_path, fname);
 		if (chdir(dir) < 0) {
 			mail_storage_set_critical(&mbox->storage->storage,
 				"chdir(%s) failed: %m", dir);
@@ -285,7 +286,7 @@ static int mbox_dotlock_privileged_op(struct mbox_mailbox *mbox,
 	if (op == MBOX_DOTLOCK_OP_LOCK) {
 		if (access(fname, R_OK) < 0) {
 			mail_storage_set_critical(&mbox->storage->storage,
-				"access(%s) failed: %m", mbox->box.path);
+				"access(%s) failed: %m", box_path);
 			return -1;
 		}
 	}
@@ -417,7 +418,7 @@ mbox_lock_dotlock_int(struct mbox_lock_context *ctx, int lock_type, bool try)
 	set.callback = dotlock_callback;
 	set.context = ctx;
 
-	ret = file_dotlock_create(&set, mbox->box.path, 0,
+	ret = file_dotlock_create(&set, mailbox_get_path(&mbox->box), 0,
 				  &mbox->mbox_dotlock);
 	if (ret >= 0) {
 		/* success / timeout */
@@ -427,7 +428,7 @@ mbox_lock_dotlock_int(struct mbox_lock_context *ctx, int lock_type, bool try)
 		ret = mbox_dotlock_privileged_op(mbox, &set,
 						 MBOX_DOTLOCK_OP_LOCK);
 	} else if (errno == EACCES)
-		mbox_dotlock_log_eacces_error(mbox, mbox->box.path);
+		mbox_dotlock_log_eacces_error(mbox, mailbox_get_path(&mbox->box));
 	else
 		mbox_set_syscall_error(mbox, "file_dotlock_create()");
 
@@ -637,7 +638,7 @@ static int mbox_lock_fcntl(struct mbox_lock_context *ctx, int lock_type,
 			mail_storage_set_critical(&ctx->mbox->storage->storage,
 				"fcntl() failed with mbox file %s: "
 				"File is locked by another process (EACCES)",
-				ctx->mbox->box.path);
+				mailbox_get_path(&ctx->mbox->box));
 			return -1;
 		}
 
@@ -709,7 +710,7 @@ static int mbox_update_locking(struct mbox_mailbox *mbox, int lock_type,
 
 	if (mbox->mbox_fd == -1 && mbox->mbox_file_stream != NULL) {
 		/* read-only mbox stream. no need to lock. */
-		i_assert(mbox->box.backend_readonly);
+		i_assert(mbox_is_backend_readonly(mbox));
 		mbox->mbox_lock_type = lock_type;
 		return 1;
 	}
@@ -774,7 +775,7 @@ static int mbox_update_locking(struct mbox_mailbox *mbox, int lock_type,
 int mbox_lock(struct mbox_mailbox *mbox, int lock_type,
 	      unsigned int *lock_id_r)
 {
-	const char *path = mbox->box.path;
+	const char *path = mailbox_get_path(&mbox->box);
 	int mbox_fd = mbox->mbox_fd;
 	bool fcntl_locked;
 	int ret;
@@ -783,8 +784,10 @@ int mbox_lock(struct mbox_mailbox *mbox, int lock_type,
 	i_assert(lock_type == F_RDLCK || lock_type == F_WRLCK);
 	i_assert(lock_type == F_RDLCK || mbox->mbox_lock_type != F_RDLCK);
 
-	/* mbox must be locked before index */
-	i_assert(mbox->box.index->lock_type != F_WRLCK);
+	/* mbox must be locked before index (the NULL check is for
+	   MAILBOX_FLAG_KEEP_LOCKED) */
+	i_assert(mbox->box.index == NULL ||
+		 mbox->box.index->lock_type != F_WRLCK);
 
 	if (mbox->mbox_lock_type == F_UNLCK) {
 		ret = mbox_update_locking(mbox, lock_type, &fcntl_locked);
