@@ -7,12 +7,12 @@
 #include <sys/stat.h>
 
 enum mbox_sync_flags {
-	MBOX_SYNC_LAST_COMMIT	= 0x01,
 	MBOX_SYNC_HEADER	= 0x02,
 	MBOX_SYNC_LOCK_READING	= 0x04,
 	MBOX_SYNC_UNDIRTY	= 0x08,
 	MBOX_SYNC_REWRITE	= 0x10,
-	MBOX_SYNC_FORCE_SYNC	= 0x20
+	MBOX_SYNC_FORCE_SYNC	= 0x20,
+	MBOX_SYNC_READONLY	= 0x40
 };
 
 struct mbox_flag_type {
@@ -138,6 +138,8 @@ struct mbox_sync_context {
 	unsigned int first_mail_crlf_expunged:1;
 
 	/* global flags: */
+	unsigned int keep_recent:1;
+	unsigned int readonly:1;
 	unsigned int delay_writes:1;
 	unsigned int renumber_uids:1;
 	unsigned int moved_offsets:1;
@@ -177,5 +179,6 @@ void mbox_sync_move_buffer(struct mbox_sync_mail_context *ctx,
 			   size_t pos, size_t need, size_t have);
 void mbox_sync_headers_add_space(struct mbox_sync_mail_context *ctx,
 				 size_t size);
+int mbox_sync_get_guid(struct mbox_mailbox *mbox);
 
 #endif
