@@ -15,6 +15,7 @@ struct maildir_mailbox_list {
 	struct mailbox_list list;
 
 	const char *global_temp_prefix, *temp_prefix;
+	char sep;
 };
 
 struct mailbox_list_iterate_context *
@@ -27,7 +28,6 @@ maildir_list_iter_next(struct mailbox_list_iterate_context *ctx);
 int maildir_list_get_mailbox_flags(struct mailbox_list *list,
 				   const char *dir, const char *fname,
 				   enum mailbox_list_file_type type,
-				   struct stat *st_r,
 				   enum mailbox_info_flags *flags);
 
 #endif
