@@ -1,4 +1,4 @@
-/* Copyright (c) 2004-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2004-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "istream.h"
@@ -29,7 +29,7 @@ mail_storage_try_copy(struct mail_save_context **_ctx, struct mail *mail)
 	const char *from_envelope, *guid;
 	time_t received_date;
 
-	ctx->copying = TRUE;
+	ctx->copying_via_save = TRUE;
 
 	/* we need to open the file in any case. caching metadata is unlikely
 	   to help anything. */
@@ -87,7 +87,7 @@ int mail_storage_copy(struct mail_save_context *ctx, struct mail *mail)
 		/* keywords gets unreferenced twice: first in
 		   mailbox_save_cancel()/_finish() and second time in
 		   mailbox_copy(). */
-		mailbox_keywords_ref(ctx->transaction->box, ctx->keywords);
+		mailbox_keywords_ref(ctx->keywords);
 	}
 
 	if (mail_storage_try_copy(&ctx, mail) < 0) {
@@ -101,7 +101,19 @@ int mail_storage_copy(struct mail_save_context *ctx, struct mail *mail)
 bool mail_storage_copy_can_use_hardlink(struct mailbox *src,
 					struct mailbox *dest)
 {
-	return src->file_create_mode == dest->file_create_mode &&
-		src->file_create_gid == dest->file_create_gid &&
+	const struct mailbox_permissions *src_perm =
+		mailbox_get_permissions(src);
+	const struct mailbox_permissions *dest_perm =
+		mailbox_get_permissions(dest);
+
+	if (src_perm->file_uid != dest_perm->file_uid) {
+		/* if we don't have read permissions, we can't hard link
+		   (basically we'll catch 0600 files here) */
+		if ((src_perm->file_create_mode & 0022) == 0)
+			return FALSE;
+	}
+
+	return src_perm->file_create_mode == dest_perm->file_create_mode &&
+		src_perm->file_create_gid == dest_perm->file_create_gid &&
 		!dest->disable_reflink_copy_to;
 }
