@@ -765,6 +765,10 @@ int mailbox_exists(struct mailbox *box, bool auto_boxes,
 		return 0;
 	}
 
+	/* if this is a shared namespace with only INBOX and
+	   mail_shared_explicit_inbox=no, we'll need to mark the namespace as
+	   usable here since nothing else will. */
+	box->list->ns->flags |= NAMESPACE_FLAG_USABLE;
 	return 0;
 }
 
@@ -914,9 +918,37 @@ static int mailbox_open_full(struct mailbox *box, struct istream *input)
 	return 0;
 }
 
+static bool mailbox_try_undelete(struct mailbox *box)
+{
+	time_t mtime;
+
+	if (mail_index_get_modification_time(box->index, &mtime) < 0)
+		return FALSE;
+	if (mtime + MAILBOX_DELETE_RETRY_SECS > time(NULL))
+		return FALSE;
+
+	if (mailbox_mark_index_deleted(box, FALSE) < 0)
+		return FALSE;
+	box->mailbox_deleted = FALSE;
+	return TRUE;
+}
+
 int mailbox_open(struct mailbox *box)
 {
-	return mailbox_open_full(box, NULL);
+	if (mailbox_open_full(box, NULL) < 0) {
+		if (!box->mailbox_deleted)
+			return -1;
+
+		/* mailbox has been marked as deleted. if this deletion
+		   started (and crashed) a long time ago, it can be confusing
+		   to user that the mailbox can't be opened. so we'll just
+		   undelete it and reopen. */
+		if(!mailbox_try_undelete(box))
+			return -1;
+		if (mailbox_open_full(box, NULL) < 0)
+			return -1;
+	}
+	return 0;
 }
 
 int mailbox_open_stream(struct mailbox *box, struct istream *input)
@@ -961,6 +993,8 @@ void mailbox_free(struct mailbox **_box)
 
 	DLLIST_REMOVE(&box->storage->mailboxes, box);
 	mail_storage_obj_unref(box->storage);
+	if (box->metadata_pool != NULL)
+		pool_unref(&box->metadata_pool);
 	pool_unref(&box->pool);
 }
 
@@ -1043,21 +1077,6 @@ int mailbox_mark_index_deleted(struct mailbox *box, bool del)
 	return 0;
 }
 
-static bool mailbox_try_undelete(struct mailbox *box)
-{
-	time_t mtime;
-
-	if (mail_index_get_modification_time(box->index, &mtime) < 0)
-		return FALSE;
-	if (mtime + MAILBOX_DELETE_RETRY_SECS > time(NULL))
-		return FALSE;
-
-	if (mailbox_mark_index_deleted(box, FALSE) < 0)
-		return FALSE;
-	box->mailbox_deleted = FALSE;
-	return TRUE;
-}
-
 int mailbox_delete(struct mailbox *box)
 {
 	int ret;
@@ -1077,19 +1096,7 @@ int mailbox_delete(struct mailbox *box)
 	if (mailbox_open(box) < 0) {
 		if (mailbox_get_last_mail_error(box) != MAIL_ERROR_NOTFOUND)
 			return -1;
-		if (!box->mailbox_deleted) {
-			/* \noselect mailbox */
-		} else {
-			/* if deletion happened a long time ago, it means it
-			   crashed while doing it. undelete the mailbox in
-			   that case. */
-			if (!mailbox_try_undelete(box))
-				return -1;
-
-			/* retry */
-			if (mailbox_open(box) < 0)
-				return -1;
-		}
+		/* \noselect mailbox */
 	}
 
 	ret = box->v.delete(box);
@@ -1105,6 +1112,18 @@ int mailbox_delete(struct mailbox *box)
 	return ret;
 }
 
+int mailbox_delete_empty(struct mailbox *box)
+{
+	int ret;
+
+	/* FIXME: should be a parameter to delete(), but since it changes API
+	   don't do it for now */
+	box->deleting_must_be_empty = TRUE;
+	ret = mailbox_delete(box);
+	box->deleting_must_be_empty = FALSE;
+	return ret;
+}
+
 static bool
 mail_storages_rename_compatible(struct mail_storage *storage1,
 				struct mail_storage *storage2,
@@ -1269,6 +1288,9 @@ int mailbox_get_metadata(struct mailbox *box, enum mailbox_metadata_items items,
 {
 	memset(metadata_r, 0, sizeof(*metadata_r));
 
+	if (box->metadata_pool != NULL)
+		p_clear(box->metadata_pool);
+
 	if (box->v.get_metadata(box, items, metadata_r) < 0)
 		return -1;
 
@@ -1327,6 +1349,8 @@ int mailbox_sync_deinit(struct mailbox_sync_context **_ctx,
 			i_error("Syncing INBOX failed: %s", errormsg);
 		}
 	}
+	if (ret == 0)
+		box->synced = TRUE;
 	return ret;
 }
 
@@ -1588,6 +1612,14 @@ void mailbox_save_set_pop3_uidl(struct mail_save_context *ctx, const char *uidl)
 	ctx->pop3_uidl = i_strdup(uidl);
 }
 
+void mailbox_save_set_pop3_order(struct mail_save_context *ctx,
+				 unsigned int order)
+{
+	i_assert(order > 0);
+
+	ctx->pop3_order = order;
+}
+
 void mailbox_save_set_dest_mail(struct mail_save_context *ctx,
 				struct mail *mail)
 {
@@ -1604,7 +1636,8 @@ int mailbox_save_begin(struct mail_save_context **ctx, struct istream *input)
 		return -1;
 	}
 
-	(*ctx)->saving = TRUE;
+	if (!(*ctx)->copying_via_save)
+		(*ctx)->saving = TRUE;
 	if (box->v.save_begin == NULL) {
 		mail_storage_set_error(box->storage, MAIL_ERROR_NOTPOSSIBLE,
 				       "Saving messages not supported");
@@ -1667,7 +1700,7 @@ int mailbox_copy(struct mail_save_context **_ctx, struct mail *mail)
 
 	if (mail_index_is_deleted(box->index)) {
 		mailbox_set_deleted(box);
-		mailbox_save_cancel(_ctx);
+		mailbox_save_cancel(&ctx);
 		return -1;
 	}
 
