@@ -50,6 +50,7 @@ struct mail_user *mail_user_alloc(const char *username,
 	user->set_info = set_info;
 	user->unexpanded_set = settings_dup(set_info, set, pool);
 	user->set = settings_dup(set_info, set, pool);
+	user->service = master_service_get_name(master_service);
 
 	/* check settings so that the duplicated structure will again
 	   contain the parsed fields */
@@ -95,25 +96,28 @@ int mail_user_init(struct mail_user *user, const char **error_r)
 {
 	const struct mail_storage_settings *mail_set;
 	const char *home, *key, *value;
+	bool need_home_dir;
 
-	if (user->_home == NULL &&
-	    settings_vars_have_key(user->set_info, user->set,
-				   'h', "home", &key, &value) &&
-	    mail_user_get_home(user, &home) <= 0) {
+	need_home_dir = user->_home == NULL &&
+		settings_vars_have_key(user->set_info, user->set,
+				       'h', "home", &key, &value);
+
+	/* expand mail_home setting before calling mail_user_get_home() */
+	settings_var_expand(user->set_info, user->set,
+			    user->pool, mail_user_var_expand_table(user));
+
+	if (need_home_dir && mail_user_get_home(user, &home) <= 0) {
 		*error_r = t_strdup_printf(
 			"userdb didn't return a home directory, "
 			"but %s used it (%%h): %s", key, value);
 		return -1;
 	}
 
-	settings_var_expand(user->set_info, user->set,
-			    user->pool, mail_user_var_expand_table(user));
 	if (mail_user_expand_plugins_envs(user, error_r) < 0)
 		return -1;
 
 	mail_set = mail_user_set_get_storage_set(user);
 	user->mail_debug = mail_set->mail_debug;
-	user->service = master_service_get_name(master_service);
 
 	user->initialized = TRUE;
 	hook_mail_user_created(user);
@@ -166,6 +170,8 @@ void mail_user_set_vars(struct mail_user *user, const char *service,
 			const struct ip_addr *local_ip,
 			const struct ip_addr *remote_ip)
 {
+	i_assert(service != NULL);
+
 	user->service = p_strdup(user->pool, service);
 	if (local_ip != NULL && local_ip->family != 0) {
 		user->local_ip = p_new(user->pool, struct ip_addr, 1);
@@ -263,7 +269,7 @@ const char *mail_user_home_expand(struct mail_user *user, const char *path)
 	return path;
 }
 
-int mail_user_get_home(struct mail_user *user, const char **home_r)
+static int mail_user_userdb_lookup_home(struct mail_user *user)
 {
 	struct auth_user_info info;
 	struct auth_user_reply reply;
@@ -271,19 +277,15 @@ int mail_user_get_home(struct mail_user *user, const char **home_r)
 	const char *username, *const *fields;
 	int ret;
 
+	i_assert(!user->home_looked_up);
+
 	memset(&info, 0, sizeof(info));
-	info.service = "lib-storage";
+	info.service = user->service;
 	if (user->local_ip != NULL)
 		info.local_ip = *user->local_ip;
 	if (user->remote_ip != NULL)
 		info.remote_ip = *user->remote_ip;
 
-	if (user->home_looked_up) {
-		*home_r = user->_home;
-		return user->_home != NULL ? 1 : 0;
-	}
-	*home_r = NULL;
-
 	if (mail_user_auth_master_conn == NULL)
 		return 0;
 
@@ -291,18 +293,37 @@ int mail_user_get_home(struct mail_user *user, const char **home_r)
 	ret = auth_master_user_lookup(mail_user_auth_master_conn,
 				      user->username, &info, userdb_pool,
 				      &username, &fields);
-	if (ret >= 0) {
+	if (ret > 0) {
 		auth_user_fields_parse(fields, userdb_pool, &reply);
-		user->_home = ret == 0 ? NULL :
-			p_strdup(user->pool, reply.home);
-		user->home_looked_up = TRUE;
-		ret = user->_home != NULL ? 1 : 0;
-		*home_r = user->_home;
+		user->_home = p_strdup(user->pool, reply.home);
 	}
 	pool_unref(&userdb_pool);
 	return ret;
 }
 
+int mail_user_get_home(struct mail_user *user, const char **home_r)
+{
+	int ret;
+
+	if (user->home_looked_up) {
+		*home_r = user->_home;
+		return user->_home != NULL ? 1 : 0;
+	}
+
+	ret = mail_user_userdb_lookup_home(user);
+	if (ret < 0)
+		return -1;
+
+	if (ret > 0 && user->_home == NULL && *user->set->mail_home != '\0') {
+		/* no home in userdb, fallback to mail_home setting */
+		user->_home = user->set->mail_home;
+	}
+	user->home_looked_up = TRUE;
+
+	*home_r = user->_home;
+	return user->_home != NULL ? 1 : 0;
+}
+
 bool mail_user_is_plugin_loaded(struct mail_user *user, struct module *module)
 {
 	const char *const *plugins;
@@ -341,7 +362,12 @@ int mail_user_try_home_expand(struct mail_user *user, const char **pathp)
 {
 	const char *home, *path = *pathp;
 
-	if (mail_user_get_home(user, &home) < 0)
+	if (*path != '~') {
+		/* no need to expand home */
+		return 0;
+	}
+
+	if (mail_user_get_home(user, &home) <= 0)
 		return -1;
 
 	path = home_expand_tilde(path, home);
