@@ -1,8 +1,9 @@
-/* Copyright (c) 2002-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "ioloop.h"
 #include "buffer.h"
+#include "hash.h"
 #include "hex-binary.h"
 #include "crc32.h"
 #include "sha1.h"
@@ -38,7 +39,14 @@ void mail_set_seq(struct mail *mail, uint32_t seq)
 {
 	struct mail_private *p = (struct mail_private *)mail;
 
-	p->v.set_seq(mail, seq);
+	p->v.set_seq(mail, seq, FALSE);
+}
+
+void mail_set_seq_saving(struct mail *mail, uint32_t seq)
+{
+	struct mail_private *p = (struct mail_private *)mail;
+
+	p->v.set_seq(mail, seq, TRUE);
 }
 
 bool mail_set_uid(struct mail *mail, uint32_t uid)
@@ -48,6 +56,22 @@ bool mail_set_uid(struct mail *mail, uint32_t uid)
 	return p->v.set_uid(mail, uid);
 }
 
+bool mail_prefetch(struct mail *mail)
+{
+	struct mail_private *p = (struct mail_private *)mail;
+
+	return p->v.prefetch(mail);
+}
+
+void mail_add_temp_wanted_fields(struct mail *mail,
+				 enum mail_fetch_field fields,
+				 struct mailbox_header_lookup_ctx *headers)
+{
+	struct mail_private *p = (struct mail_private *)mail;
+
+	p->v.add_temp_wanted_fields(mail, fields, headers);
+}
+
 enum mail_flags mail_get_flags(struct mail *mail)
 {
 	struct mail_private *p = (struct mail_private *)mail;
@@ -177,7 +201,17 @@ int mail_get_stream(struct mail *mail, struct message_size *hdr_size,
 
 	if (mail->lookup_abort != MAIL_LOOKUP_ABORT_NEVER)
 		return mail_set_aborted(mail);
-	return p->v.get_stream(mail, hdr_size, body_size, stream_r);
+	return p->v.get_stream(mail, TRUE, hdr_size, body_size, stream_r);
+}
+
+int mail_get_hdr_stream(struct mail *mail, struct message_size *hdr_size,
+			struct istream **stream_r)
+{
+	struct mail_private *p = (struct mail_private *)mail;
+
+	if (mail->lookup_abort != MAIL_LOOKUP_ABORT_NEVER)
+		return mail_set_aborted(mail);
+	return p->v.get_stream(mail, FALSE, hdr_size, NULL, stream_r);
 }
 
 int mail_get_special(struct mail *mail, enum mail_fetch_field field,
@@ -240,17 +274,11 @@ void mail_set_expunged(struct mail *mail)
 	mail->expunged = TRUE;
 }
 
-bool mail_is_cached(struct mail *mail)
-{
-	return mail_cache_field_exists_any(mail->transaction->cache_view,
-					   mail->seq);
-}
-
-void mail_parse(struct mail *mail, bool parse_body)
+void mail_precache(struct mail *mail)
 {
 	struct mail_private *p = (struct mail_private *)mail;
 
-	p->v.parse(mail, parse_body);
+	p->v.precache(mail);
 }
 
 void mail_set_cache_corrupted(struct mail *mail, enum mail_fetch_field field)
@@ -260,103 +288,21 @@ void mail_set_cache_corrupted(struct mail *mail, enum mail_fetch_field field)
 	p->v.set_cache_corrupted(mail, field);
 }
 
-const char *mail_generate_guid_string(void)
-{
-	static struct timespec ts = { 0, 0 };
-	static unsigned int pid = 0;
-
-	/* we'll use the current time in nanoseconds as the initial 64bit
-	   counter. */
-	if (ts.tv_sec == 0) {
-		if (clock_gettime(CLOCK_REALTIME, &ts) < 0)
-			i_fatal("clock_gettime() failed: %m");
-		pid = getpid();
-	} else if ((uint32_t)ts.tv_nsec < (uint32_t)-1) {
-		ts.tv_nsec++;
-	} else {
-		ts.tv_sec++;
-		ts.tv_nsec = 0;
-	}
-	return t_strdup_printf("%04x%04lx%04x%s",
-			       (unsigned int)ts.tv_nsec,
-			       (unsigned long)ts.tv_sec,
-			       pid, my_hostname);
-}
-
-void mail_generate_guid_128(uint8_t guid[MAIL_GUID_128_SIZE])
-{
-	static struct timespec ts = { 0, 0 };
-	static uint8_t guid_static[8];
-	uint32_t pid, host_crc;
-
-	/* we'll use the current time in nanoseconds as the initial 64bit
-	   counter. */
-	if (ts.tv_sec == 0) {
-		if (clock_gettime(CLOCK_REALTIME, &ts) < 0)
-			i_fatal("clock_gettime() failed: %m");
-		pid = getpid();
-		host_crc = crc32_str(my_hostname);
-
-		guid_static[0] = (pid & 0x000000ff);
-		guid_static[1] = (pid & 0x0000ff00) >> 8;
-		guid_static[2] = (pid & 0x00ff0000) >> 16;
-		guid_static[3] = (pid & 0xff000000) >> 24;
-		guid_static[4] = (host_crc & 0x000000ff);
-		guid_static[5] = (host_crc & 0x0000ff00) >> 8;
-		guid_static[6] = (host_crc & 0x00ff0000) >> 16;
-		guid_static[7] = (host_crc & 0xff000000) >> 24;
-	} else if ((uint32_t)ts.tv_nsec < (uint32_t)-1) {
-		ts.tv_nsec++;
-	} else {
-		ts.tv_sec++;
-		ts.tv_nsec = 0;
-	}
-
-	guid[0] = (ts.tv_nsec & 0x000000ff);
-	guid[1] = (ts.tv_nsec & 0x0000ff00) >> 8;
-	guid[2] = (ts.tv_nsec & 0x00ff0000) >> 16;
-	guid[3] = (ts.tv_nsec & 0xff000000) >> 24;
-	guid[4] = (ts.tv_sec & 0x000000ff);
-	guid[5] = (ts.tv_sec & 0x0000ff00) >> 8;
-	guid[6] = (ts.tv_sec & 0x00ff0000) >> 16;
-	guid[7] = (ts.tv_sec & 0xff000000) >> 24;
-	memcpy(guid + 8, guid_static, 8);
-}
-
-void mail_generate_guid_128_hash(const char *guid,
-				 uint8_t guid_128[MAIL_GUID_128_SIZE])
+void mail_generate_guid_128_hash(const char *guid, guid_128_t guid_128_r)
 {
 	unsigned char sha1_sum[SHA1_RESULTLEN];
 	buffer_t buf;
 
-	buffer_create_data(&buf, guid_128, MAIL_GUID_128_SIZE);
-	if (strlen(guid) != MAIL_GUID_128_SIZE*2 ||
-	    hex_to_binary(guid, &buf) < 0 ||
-	    buf.used != MAIL_GUID_128_SIZE) {
+	if (guid_128_from_string(guid, guid_128_r) < 0) {
 		/* not 128bit hex. use a hash of it instead. */
+		buffer_create_data(&buf, guid_128_r, GUID_128_SIZE);
 		buffer_set_used_size(&buf, 0);
 		sha1_get_digest(guid, strlen(guid), sha1_sum);
 #if SHA1_RESULTLEN < DBOX_GUID_BIN_LEN
 #  error not possible
 #endif
 		buffer_append(&buf,
-			      sha1_sum + SHA1_RESULTLEN - MAIL_GUID_128_SIZE,
-			      MAIL_GUID_128_SIZE);
+			      sha1_sum + SHA1_RESULTLEN - GUID_128_SIZE,
+			      GUID_128_SIZE);
 	}
 }
-
-bool mail_guid_128_is_empty(const uint8_t guid_128[MAIL_GUID_128_SIZE])
-{
-	unsigned int i;
-
-	for (i = 0; i < MAIL_GUID_128_SIZE; i++) {
-		if (guid_128[i] != 0)
-			return FALSE;
-	}
-	return TRUE;
-}
-
-const char *mail_guid_128_to_string(const uint8_t guid_128[MAIL_GUID_128_SIZE])
-{
-	return binary_to_hex(guid_128, MAIL_GUID_128_SIZE);
-}
