@@ -27,4 +27,18 @@ bool bsearch_insert_pos(const void *key, const void *base, unsigned int nmemb,
 			size_t size, int (*cmp)(const void *, const void *),
 			unsigned int *idx_r);
 
+bool array_bsearch_insert_pos_i(const struct array *array, const void *key,
+				int (*cmp)(const void *, const void *),
+				unsigned int *idx_r);
+#ifdef CONTEXT_TYPE_SAFETY
+#define array_bsearch_insert_pos(array, key, cmp, idx_r) \
+	({(void)(1 ? 0 : cmp(key, ARRAY_TYPE_CAST_CONST(array)NULL)); \
+	array_bsearch_insert_pos_i(&(array)->arr, (const void *)key, \
+		(int (*)(const void *, const void *))cmp, idx_r); })
+#else
+#define array_bsearch_insert_pos(array, key, cmp, idx_r) \
+	array_bsearch_insert_pos_i(&(array)->arr, (const void *)key, \
+		(int (*)(const void *, const void *))cmp, idx_r)
+#endif
+
 #endif
