@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2006-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "istream.h"
@@ -69,7 +69,7 @@ static int file_copy_to_tmp(const char *srcpath, const char *tmppath,
 	if (fchown(fd_out, (uid_t)-1, st.st_gid) < 0 && errno != EPERM)
 		i_error("fchown(%s) failed: %m", tmppath);
 
-	input = i_stream_create_fd(fd_in, 0, FALSE);
+	input = i_stream_create_fd(fd_in, IO_BLOCK_SIZE, FALSE);
 	output = o_stream_create_fd_file(fd_out, 0, FALSE);
 
 	while ((ret = o_stream_send_istream(output, input)) > 0) ;
