@@ -18,9 +18,6 @@
 #define N_ELEMENTS(arr) \
 	(sizeof(arr) / sizeof((arr)[0]))
 
-#define BITS_IN_UINT (CHAR_BIT * sizeof(unsigned int))
-#define BITS_IN_SIZE_T (CHAR_BIT * sizeof(size_t))
-
 #define MEM_ALIGN(size) \
 	(((size) + MEM_ALIGN_SIZE-1) & ~((unsigned int) MEM_ALIGN_SIZE-1))
 
@@ -34,13 +31,6 @@
 #define I_MIN(a, b)  (((a) < (b)) ? (a) : (b))
 #define I_MAX(a, b)  (((a) > (b)) ? (a) : (b))
 
-#undef CLAMP
-#define CLAMP(x, low, high) \
-	(((x) > (high)) ? (high) : (((x) < (low)) ? (low) : (x)))
-
-#undef NVL
-#define NVL(str, nullstr) ((str) != NULL ? (str) : (nullstr))
-
 /* make it easier to cast from/to pointers. assumes that
    sizeof(size_t) == sizeof(void *) and they're both the largest datatypes
    that are allowed to be used. so, long long isn't safe with these. */
@@ -67,8 +57,8 @@
  */
 #define STRUCT_MEMBER_P(struct_p, struct_offset) \
 	((void *) ((char *) (struct_p) + (long) (struct_offset)))
-#define STRUCT_MEMBER(member_type, struct_p, struct_offset) \
-	(*(member_type *) G_STRUCT_MEMBER_P((struct_p), (struct_offset)))
+#define CONST_STRUCT_MEMBER_P(struct_p, struct_offset) \
+	((const void *) ((const char *) (struct_p) + (long) (struct_offset)))
 
 /* Provide simple macro statement wrappers (adapted from Perl):
    STMT_START { statements; } STMT_END;
@@ -103,6 +93,8 @@
 	__attribute__((format_arg (arg_idx)))
 #  define ATTR_SCANF(format_idx, arg_idx) \
 	__attribute__((format (scanf, format_idx, arg_idx)))
+#  define ATTR_STRFTIME(format_idx) \
+	__attribute__((format (strftime, format_idx, 0)))
 #  define ATTR_UNUSED __attribute__((unused))
 #  define ATTR_NORETURN __attribute__((noreturn))
 #  define ATTR_CONST __attribute__((const))
@@ -110,7 +102,8 @@
 #else
 #  define ATTR_FORMAT(format_idx, arg_idx)
 #  define ATTR_FORMAT_ARG(arg_idx)
-#  define ATTR_SCANF
+#  define ATTR_SCANF(format_idx, arg_idx)
+#  define ATTR_STRFTIME(format_idx)
 #  define ATTR_UNUSED
 #  define ATTR_NORETURN
 #  define ATTR_CONST
@@ -129,13 +122,13 @@
 #  define ATTR_WARN_UNUSED_RESULT
 #  define ATTR_SENTINEL
 #endif
-
-/* C99-style struct member definitions */
-#if (defined(__STDC__) && __STDC_VERSION__ >= 199901L && \
-	!defined(_HPUX_SOURCE)) || __GNUC__ > 2
-#  define MEMBER(name) .name =
+#if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
+/* GCC 4.3 and later */
+#  define ATTR_HOT __attribute__((hot))
+#  define ATTR_COLD __attribute__((cold))
 #else
-#  define MEMBER(name)
+#  define ATTR_HOT
+#  define ATTR_COLD
 #endif
 
 /* Macros to provide type safety for callback functions' context parameters */
@@ -146,9 +139,14 @@
 #  define CONTEXT_CALLBACK(name, callback_type, callback, context, ...) \
 	({(void)(1 ? 0 : callback(context)); \
 	name(__VA_ARGS__, (callback_type *)callback, context); })
+#  define CONTEXT_CALLBACK2(name, callback_type, callback, arg1_type, context, ...) \
+	({(void)(1 ? 0 : callback((arg1_type)0, context)); \
+	name(__VA_ARGS__, (callback_type *)callback, context); })
 #else
 #  define CONTEXT_CALLBACK(name, callback_type, callback, context, ...) \
 	name(__VA_ARGS__, (callback_type *)callback, context)
+#  define CONTEXT_CALLBACK2(name, callback_type, callback, arg1_type, context, ...) \
+	name(__VA_ARGS__, (callback_type *)callback, context)
 #endif
 
 #if __GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ > 0)
@@ -163,16 +161,6 @@
 #  define COMPILE_ERROR_IF_TYPES_NOT_COMPATIBLE(_a, _b) 0
 #endif
 
-/* Wrap the gcc __PRETTY_FUNCTION__ and __FUNCTION__ variables with
-   macros, so we can refer to them as strings unconditionally. */
-#ifdef __GNUC__
-#  define GNUC_FUNCTION __FUNCTION__
-#  define GNUC_PRETTY_FUNCTION __PRETTY_FUNCTION__
-#else
-#  define GNUC_FUNCTION ""
-#  define GNUC_PRETTY_FUNCTION ""
-#endif
-
 #if __GNUC__ > 2
 #  define unlikely(expr) __builtin_expect(!!(expr), 0)
 #  define likely(expr) __builtin_expect(!!(expr), 1)
@@ -191,7 +179,7 @@
        i_panic("file %s: line %d (%s): assertion failed: (%s)",		\
 		__FILE__,						\
 		__LINE__,						\
-		__PRETTY_FUNCTION__,					\
+		__FUNCTION__,					\
 		#expr);			}STMT_END
 
 #else /* !__GNUC__ */
