@@ -31,6 +31,11 @@ struct ip_addr {
 };
 ARRAY_DEFINE_TYPE(ip_addr, struct ip_addr);
 
+struct net_unix_cred {
+	uid_t uid;
+	gid_t gid;
+};
+
 /* maxmimum string length of IP address */
 #ifdef HAVE_IPV6
 #  define MAX_IP_LEN INET6_ADDRSTRLEN
@@ -40,6 +45,7 @@ ARRAY_DEFINE_TYPE(ip_addr, struct ip_addr);
 
 #define IPADDR_IS_V4(ip) ((ip)->family == AF_INET)
 #define IPADDR_IS_V6(ip) ((ip)->family == AF_INET6)
+#define IPADDR_BITS(ip) (IPADDR_IS_V4(ip) ? 32 : 128)
 
 /* Returns TRUE if IPs are the same */
 bool net_ip_compare(const struct ip_addr *ip1, const struct ip_addr *ip2);
@@ -109,6 +115,8 @@ int net_getsockname(int fd, struct ip_addr *addr, unsigned int *port);
 int net_getpeername(int fd, struct ip_addr *addr, unsigned int *port);
 /* Get UNIX socket name. */
 int net_getunixname(int fd, const char **name_r);
+/* Get UNIX socket peer process's credentials. */
+int net_getunixcred(int fd, struct net_unix_cred *cred_r);
 
 /* Returns ip_addr as string, or NULL if ip is invalid. */
 const char *net_ip2addr(const struct ip_addr *ip);
