@@ -27,6 +27,10 @@ void var_expand_with_funcs(string_t *dest, const char *str,
 /* Returns the actual key character for given string, ie. skip any modifiers
    that are before it. The string should be the data after the '%' character. */
 char var_get_key(const char *str) ATTR_PURE;
+/* Similar to var_get_key(), but works for long keys as well. For single char
+   keys size=1, while for e.g. %{key} size=3 and idx points to 'k'. */
+void var_get_key_range(const char *str, unsigned int *idx_r,
+		       unsigned int *size_r);
 /* Returns TRUE if key variable is used in the string. long_key may be NULL. */
 bool var_has_key(const char *str, char key, const char *long_key) ATTR_PURE;
 
