@@ -1,7 +1,8 @@
 #ifndef LOG_CONNECTION_H
 #define LOG_CONNECTION_H
 
-struct log_connection *log_connection_create(int fd, int listen_fd);
+struct log_connection *
+log_connection_create(struct log_error_buffer *errorbuf, int fd, int listen_fd);
 void log_connection_destroy(struct log_connection *log);
 
 void log_connections_init(void);
