@@ -10,6 +10,7 @@ bool ssl_initialized = FALSE;
 /* no SSL support */
 
 int ssl_proxy_alloc(int fd ATTR_UNUSED, const struct ip_addr *ip ATTR_UNUSED,
+		    pool_t set_pool ATTR_UNUSED,
 		    const struct login_settings *set ATTR_UNUSED,
 		    struct ssl_proxy **proxy_r ATTR_UNUSED)
 {
@@ -18,6 +19,7 @@ int ssl_proxy_alloc(int fd ATTR_UNUSED, const struct ip_addr *ip ATTR_UNUSED,
 }
 
 int ssl_proxy_client_alloc(int fd ATTR_UNUSED, struct ip_addr *ip ATTR_UNUSED,
+			   pool_t set_pool ATTR_UNUSED,
 			   const struct login_settings *set ATTR_UNUSED,
 			   ssl_handshake_callback_t *callback ATTR_UNUSED,
 			   void *context ATTR_UNUSED,
@@ -77,6 +79,11 @@ const char *ssl_proxy_get_compression(struct ssl_proxy *proxy ATTR_UNUSED)
 	return NULL;
 }
 
+const char *ssl_proxy_get_cert_error(struct ssl_proxy *proxy ATTR_UNUSED)
+{
+	return "";
+}
+
 void ssl_proxy_free(struct ssl_proxy **proxy ATTR_UNUSED) {}
 
 unsigned int ssl_proxy_get_count(void)
