@@ -1,4 +1,4 @@
-/* Copyright (c) 2005-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2005-2012 Dovecot authors, see the included COPYING file */
 
 #include "common.h"
 #include "array.h"
@@ -20,9 +20,10 @@
 #include <syslog.h>
 #include <signal.h>
 
-#define SERVICE_STARTUP_FAILURE_THROTTLE_SECS 60
 #define SERVICE_DROP_WARN_INTERVAL_SECS 60
 #define SERVICE_DROP_TIMEOUT_MSECS (10*1000)
+#define MAX_DIE_WAIT_SECS 5
+#define SERVICE_MAX_EXIT_FAILURES_IN_SEC 10
 
 static void service_monitor_start_extra_avail(struct service *service);
 static void service_status_more(struct service_process *process,
@@ -171,8 +172,10 @@ static void service_status_input(struct service *service)
 	if (ret <= 0) {
 		if (ret == 0)
 			service_error(service, "read(status) failed: EOF");
-		else
+		else if (errno != EAGAIN)
 			service_error(service, "read(status) failed: %m");
+		else
+			return;
 		service_monitor_stop(service);
 		return;
 	}
@@ -193,8 +196,14 @@ static void service_monitor_throttle(struct service *service)
 	if (service->to_throttle != NULL)
 		return;
 
-	service_error(service, "command startup failed, throttling");
-	service_throttle(service, SERVICE_STARTUP_FAILURE_THROTTLE_SECS);
+	i_assert(service->throttle_secs > 0);
+
+	service_error(service, "command startup failed, throttling for %u secs",
+		      service->throttle_secs);
+	service_throttle(service, service->throttle_secs);
+	service->throttle_secs *= 2;
+	if (service->throttle_secs > SERVICE_STARTUP_FAILURE_THROTTLE_MAX_SECS)
+		service->throttle_secs = SERVICE_STARTUP_FAILURE_THROTTLE_MAX_SECS;
 }
 
 static void service_drop_timeout(struct service *service)
@@ -289,7 +298,8 @@ static void service_monitor_start_extra_avail(struct service *service)
 {
 	unsigned int i, count;
 
-	if (service->process_avail >= service->set->process_min_avail)
+	if (service->process_avail >= service->set->process_min_avail ||
+	    service->list->destroying)
 		return;
 
 	count = service->set->process_min_avail - service->process_avail;
@@ -388,7 +398,8 @@ void services_monitor_start(struct service_list *service_list)
 {
 	struct service *const *services;
 
-	services_log_init(service_list);
+	if (services_log_init(service_list) < 0)
+		return;
 	service_anvil_monitor_start(service_list);
 
 	if (pipe(service_list->master_dead_pipe_fd) < 0)
@@ -424,14 +435,17 @@ void services_monitor_start(struct service_list *service_list)
 		service_monitor_listen_start(service);
 	}
 
-	if (service_process_create(service_list->log) != NULL)
-		service_monitor_listen_stop(service_list->log);
+	if (service_list->log->status_fd[0] != -1) {
+		if (service_process_create(service_list->log) != NULL)
+			service_monitor_listen_stop(service_list->log);
+	}
 
 	/* start up a process for startup-services */
 	array_foreach(&service_list->services, services) {
 		struct service *service = *services;
 
-		if (service->type == SERVICE_TYPE_STARTUP) {
+		if (service->type == SERVICE_TYPE_STARTUP &&
+		    service->status_fd[0] != -1) {
 			if (service_process_create(service) != NULL)
 				service_monitor_listen_stop(service);
 		}
@@ -470,7 +484,28 @@ void service_monitor_stop(struct service *service)
 		timeout_remove(&service->to_throttle);
 }
 
-void services_monitor_stop(struct service_list *service_list)
+static void services_monitor_wait(struct service_list *service_list)
+{
+	struct service *const *servicep;
+	time_t max_wait_time = time(NULL) + MAX_DIE_WAIT_SECS;
+	bool finished;
+
+	for (;;) {
+		finished = TRUE;
+		services_monitor_reap_children();
+		array_foreach(&service_list->services, servicep) {
+			if ((*servicep)->status_fd[0] != -1)
+				service_status_input(*servicep);
+			if ((*servicep)->process_avail > 0)
+				finished = FALSE;
+		}
+		if (finished || time(NULL) > max_wait_time)
+			break;
+		usleep(100000);
+	}
+}
+
+void services_monitor_stop(struct service_list *service_list, bool wait)
 {
 	struct service *const *services;
 
@@ -483,6 +518,13 @@ void services_monitor_stop(struct service_list *service_list)
 		service_list->master_dead_pipe_fd[1] = -1;
 	}
 
+	if (wait) {
+		/* we've notified all children that the master is dead.
+		   now wait for the children to either die or to tell that
+		   they're no longer listening for new connections */
+		services_monitor_wait(service_list);
+	}
+
 	array_foreach(&service_list->services, services)
 		service_monitor_stop(*services);
 
@@ -492,10 +534,24 @@ void services_monitor_stop(struct service_list *service_list)
 static bool
 service_process_failure(struct service_process *process, int status)
 {
+	struct service *service = process->service;
 	bool throttle;
 
 	service_process_log_status_error(process, status);
-	throttle = process->total_count == 0;
+	throttle = process->to_status != NULL;
+	if (!throttle && !service->have_successful_exits) {
+		/* this service has seen no successful exits yet.
+		   try to avoid failure storms by throttling the service if it
+		   only keeps failing rapidly. this is no longer done after
+		   one success to avoid intentional DoSing, in case attacker
+		   finds a way to quickly crash his own session. */
+		if (service->exit_failure_last != ioloop_time) {
+			service->exit_failure_last = ioloop_time;
+			service->exit_failures_in_sec = 0;
+		}
+		if (++service->exit_failures_in_sec > SERVICE_MAX_EXIT_FAILURES_IN_SEC)
+			throttle = TRUE;
+	}
 	service_process_notify_add(service_anvil_global->kills, process);
 	return throttle;
 }
@@ -519,8 +575,14 @@ void services_monitor_reap_children(void)
 		service = process->service;
 		if (status == 0) {
 			/* success */
-			if (service->listen_pending)
+			if (service->listen_pending &&
+			    !service->list->destroying)
 				service_monitor_listen_start(service);
+			/* one success resets all failures */
+			service->have_successful_exits = TRUE;
+			service->exit_failures_in_sec = 0;
+			service->throttle_secs =
+				SERVICE_STARTUP_FAILURE_THROTTLE_MIN_SECS;
 			throttle = FALSE;
 		} else {
 			throttle = service_process_failure(process, status);
