@@ -1,4 +1,4 @@
-/* Copyright (c) 2008-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2008-2012 Dovecot authors, see the included COPYING file */
 
 #include "imap-common.h"
 #include "str.h"
@@ -17,9 +17,6 @@
 #define ERROR_NOT_ADMIN "["IMAP_RESP_CODE_NOPERM"] " \
 	"You lack administrator privileges on this mailbox."
 
-#define ACL_MAILBOX_FLAGS \
-	(MAILBOX_FLAG_READONLY | MAILBOX_FLAG_KEEP_RECENT)
-
 #define IMAP_ACL_ANYONE "anyone"
 #define IMAP_ACL_AUTHENTICATED "authenticated"
 #define IMAP_ACL_OWNER "owner"
@@ -57,8 +54,7 @@ acl_mailbox_open_as_admin(struct client_command_context *cmd, const char *name)
 {
 	struct mail_namespace *ns;
 	struct mailbox *box;
-	const char *storage_name;
-	enum mailbox_name_status status;
+	enum mailbox_existence existence = MAILBOX_EXISTENCE_NONE;
 	int ret;
 
 	if (ACL_USER_CONTEXT(cmd->client->user) == NULL) {
@@ -66,29 +62,24 @@ acl_mailbox_open_as_admin(struct client_command_context *cmd, const char *name)
 		return NULL;
 	}
 
-	ns = client_find_namespace(cmd, name, &storage_name, &status);
+	ns = client_find_namespace(cmd, &name);
 	if (ns == NULL)
 		return NULL;
 
-	switch (status) {
-	case MAILBOX_NAME_INVALID:
-	case MAILBOX_NAME_VALID:
-		client_fail_mailbox_name_status(cmd, name, NULL, status);
-		return NULL;
-	default:
-		break;
-	}
-
 	/* Force opening the mailbox so that we can give a nicer error message
 	   if mailbox isn't selectable but is listable. */
-	box = mailbox_alloc(ns->list, storage_name, ACL_MAILBOX_FLAGS |
+	box = mailbox_alloc(ns->list, name, MAILBOX_FLAG_READONLY |
 			    MAILBOX_FLAG_IGNORE_ACLS);
-	ret = acl_mailbox_right_lookup(box, ACL_STORAGE_RIGHT_ADMIN);
-	if (ret > 0)
-		return box;
+	if (mailbox_exists(box, TRUE, &existence) == 0 &&
+	    existence == MAILBOX_EXISTENCE_SELECT) {
+		ret = acl_mailbox_right_lookup(box, ACL_STORAGE_RIGHT_ADMIN);
+		if (ret > 0)
+			return box;
+	}
 
-	/* not an administrator. */
-	if (acl_mailbox_right_lookup(box, ACL_STORAGE_RIGHT_LOOKUP) <= 0) {
+	/* mailbox doesn't exist / not an administrator. */
+	if (existence != MAILBOX_EXISTENCE_SELECT ||
+	    acl_mailbox_right_lookup(box, ACL_STORAGE_RIGHT_LOOKUP) <= 0) {
 		client_send_tagline(cmd, t_strdup_printf(
 			"NO ["IMAP_RESP_CODE_NONEXISTENT"] "
 			MAIL_ERRSTR_MAILBOX_NOT_FOUND, name));
@@ -318,7 +309,7 @@ static bool cmd_myrights(struct client_command_context *cmd)
 {
 	struct mail_namespace *ns;
 	struct mailbox *box;
-	const char *mailbox, *storage_name;
+	const char *mailbox;
 	const char *const *rights;
 	string_t *str;
 
@@ -330,12 +321,12 @@ static bool cmd_myrights(struct client_command_context *cmd)
 		return TRUE;
 	}
 
-	ns = client_find_namespace(cmd, mailbox, &storage_name, NULL);
+	ns = client_find_namespace(cmd, &mailbox);
 	if (ns == NULL)
 		return TRUE;
 
-	box = mailbox_alloc(ns->list, storage_name,
-			    ACL_MAILBOX_FLAGS | MAILBOX_FLAG_IGNORE_ACLS);
+	box = mailbox_alloc(ns->list, mailbox,
+			    MAILBOX_FLAG_READONLY | MAILBOX_FLAG_IGNORE_ACLS);
 	if (acl_object_get_my_rights(acl_mailbox_get_aclobj(box),
 				     pool_datastack_create(), &rights) < 0) {
 		client_send_tagline(cmd, "NO "MAIL_ERRSTR_CRITICAL_MSG);
