@@ -1,7 +1,8 @@
-/* Copyright (c) 2007-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2007-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "array.h"
+#include "unichar.h"
 #include "mail-namespace.h"
 #include "mail-search-build.h"
 #include "mail-storage-private.h"
@@ -51,20 +52,19 @@ static int snarf(struct mailbox *srcbox, struct mailbox *destbox)
 
 	search_args = mail_search_build_init();
 	mail_search_build_add_all(search_args);
-	search_ctx = mailbox_search_init(src_trans, search_args, NULL);
+	search_ctx = mailbox_search_init(src_trans, search_args, NULL,
+					 MAIL_FETCH_STREAM_HEADER |
+					 MAIL_FETCH_STREAM_BODY, NULL);
 	mail_search_args_unref(&search_args);
 
 	ret = 0;
-	mail = mail_alloc(src_trans, MAIL_FETCH_STREAM_HEADER |
-			  MAIL_FETCH_STREAM_BODY, NULL);
-	while (mailbox_search_next(search_ctx, mail)) {
+	while (mailbox_search_next(search_ctx, &mail)) {
 		if (mail->expunged)
 			continue;
 
 		save_ctx = mailbox_save_alloc(dest_trans);
 		if (mailbox_copy(&save_ctx, mail) < 0 && !mail->expunged) {
-			(void)mail_storage_get_last_error(destbox->storage,
-							  &error);
+			error = mailbox_get_last_mail_error(destbox);
 			/* if we failed because of out of disk space, just
 			   move those messages we managed to move so far. */
 			if (error != MAIL_ERROR_NOSPACE)
@@ -73,7 +73,6 @@ static int snarf(struct mailbox *srcbox, struct mailbox *destbox)
 		}
 		mail_expunge(mail);
 	}
-	mail_free(&mail);
 
 	if (mailbox_search_deinit(&search_ctx) < 0)
 		ret = -1;
@@ -128,8 +127,12 @@ snarf_box_find(struct mail_user *user, struct mailbox_list **list_r,
 	snarf_name = mail_user_plugin_getenv(user, "snarf");
 	if (snarf_name == NULL)
 		return FALSE;
+	if (!uni_utf8_str_is_valid(snarf_name)) {
+		i_error("snarf: Mailbox name not UTF-8: %s", snarf_name);
+		return FALSE;
+	}
 
-	snarf_ns = mail_namespace_find(user->namespaces, &snarf_name);
+	snarf_ns = mail_namespace_find(user->namespaces, snarf_name);
 	if (snarf_ns == NULL) {
 		i_error("snarf: Namespace not found for mailbox: %s",
 			snarf_name);
@@ -161,7 +164,6 @@ static void snarf_mailbox_allocated(struct mailbox *box)
 	box->vlast = &sbox->module_ctx.super;
 
 	sbox->snarf_box = mailbox_alloc(snarf_list, snarf_name,
-					MAILBOX_FLAG_KEEP_RECENT |
 					MAILBOX_FLAG_KEEP_LOCKED);
 
 	v->sync_init = snarf_sync_init;
@@ -171,8 +173,8 @@ static void snarf_mailbox_allocated(struct mailbox *box)
 
 static struct mailbox *
 snarf_mailbox_alloc(struct mail_storage *storage,
-		    struct mailbox_list *list, const char *name,
-		    enum mailbox_flags flags)
+		    struct mailbox_list *list,
+		    const char *vname, enum mailbox_flags flags)
 {
 	struct snarf_mail_storage *sstorage = SNARF_CONTEXT(storage);
 	struct mail_namespace *ns = mailbox_list_get_namespace(list);
@@ -181,7 +183,7 @@ snarf_mailbox_alloc(struct mail_storage *storage,
 	const char *snarf_name;
 	struct stat st;
 
-	if (strcmp(name, "INBOX") == 0 &&
+	if (strcmp(vname, "INBOX") == 0 &&
 	    (ns->flags & NAMESPACE_FLAG_INBOX_USER) != 0) {
 		if (stat(sstorage->snarf_path, &st) == 0)
 			sstorage->snarfing_disabled = FALSE;
@@ -196,13 +198,13 @@ snarf_mailbox_alloc(struct mail_storage *storage,
 			if (snarf_box_find(storage->user, &snarf_list,
 					   &snarf_name)) {
 				list = snarf_list;
-				name = snarf_name;
+				vname = snarf_name;
 			}
 		}
 	}
 
 	box = sstorage->module_ctx.super.
-		mailbox_alloc(storage, list, name, flags);
+		mailbox_alloc(storage, list, vname, flags);
 	if (sstorage->snarfing_disabled) {
 		box->inbox_user = TRUE;
 		box->inbox_any = TRUE;
