@@ -1,10 +1,10 @@
-/* Copyright (c) 2010-2011 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2010-2012 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 
 #ifdef HAVE_BZLIB
 
-#include "istream-internal.h"
+#include "istream-private.h"
 #include "istream-zlib.h"
 #include <bzlib.h>
 
